Package: aelab
Type: Package
Title: Data Processing for Aquatic Ecology
Version: 1.1.3
Authors@R: person("Zhao-Jun", "Yong", role = c("cre","aut"), email = "nuannuan0425@gmail.com")
Maintainer: Zhao-Jun Yong <nuannuan0425@gmail.com>
Description: Facilitate the analysis of data related to aquatic ecology, specifically the establishment of carbon budget. 
  Currently, the package allows the below analysis.
  (i) the calculation of greenhouse gas flux based on data obtained from trace gas analyzer using the method described in Lin et al. (2024).
  (ii) the calculation of Dissolved Oxygen (DO) metabolism based on data obtained from dissolved oxygen data logger using the method described in Staehr et al. (2010). 
  Yong et al. (2024) <doi:10.5194/bg-21-5247-2024>.
  Staehr et al. (2010) <doi:10.4319/lom.2010.8.0628>.
Imports: tibble, lubridate, stats, dplyr, openxlsx, readxl, ggplot2,
        readr, tidyr, stringr, purrr, rlang, grDevices, multcompView,
        FSA, rcompanion, rnaturalearth, sf, ggspatial
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.2.3
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), rnaturalearthdata
Config/testthat/edition: 3
Depends: R (>= 4.1.0)
VignetteBuilder: knitr
LazyData: true
NeedsCompilation: no
Packaged: 2026-05-07 08:44:52 UTC; yangshaojun
Author: Zhao-Jun Yong [cre, aut]
Repository: CRAN
Date/Publication: 2026-05-07 12:10:44 UTC
Built: R 4.5.2; ; 2026-05-07 13:42:04 UTC; unix

Package: aelab
Type: Package
Title: Data Processing for Aquatic Ecology
Version: 1.1.3
Authors@R: person("Zhao-Jun", "Yong", role = c("cre","aut"), email = "nuannuan0425@gmail.com")
Maintainer: Zhao-Jun Yong <nuannuan0425@gmail.com>
Description: Facilitate the analysis of data related to aquatic ecology, specifically the establishment of carbon budget. 
  Currently, the package allows the below analysis.
  (i) the calculation of greenhouse gas flux based on data obtained from trace gas analyzer using the method described in Lin et al. (2024).
  (ii) the calculation of Dissolved Oxygen (DO) metabolism based on data obtained from dissolved oxygen data logger using the method described in Staehr et al. (2010). 
  Yong et al. (2024) <doi:10.5194/bg-21-5247-2024>.
  Staehr et al. (2010) <doi:10.4319/lom.2010.8.0628>.
Imports: tibble, lubridate, stats, dplyr, openxlsx, readxl, ggplot2,
        readr, tidyr, stringr, purrr, rlang, grDevices, multcompView,
        FSA, rcompanion, rnaturalearth, sf, ggspatial
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.2.3
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), rnaturalearthdata
Config/testthat/edition: 3
Depends: R (>= 4.1.0)
VignetteBuilder: knitr
LazyData: true
NeedsCompilation: no
Packaged: 2026-05-07 08:44:52 UTC; yangshaojun
Author: Zhao-Jun Yong [cre, aut]
Repository: CRAN
Date/Publication: 2026-05-07 12:10:44 UTC

# Generated by roxygen2: do not edit by hand

export(aelab_palettes)
export(aov_test)
export(calc_chla_trichromatic)
export(calculate_MDF)
export(calculate_do)
export(calculate_ghg_flux)
export(calculate_regression)
export(calculate_total_co2e)
export(combine_hobo)
export(combine_weather)
export(combine_weather_month)
export(convert_ghg_unit)
export(convert_time)
export(descriptive_statistic)
export(df_trans)
export(find_outlier)
export(ks_test)
export(normality_test_aov)
export(normality_test_t)
export(plot_bar)
export(plot_box)
export(plot_hobo)
export(plot_line)
export(plot_map_taiwan)
export(plot_point)
export(process_hobo)
export(process_info)
export(process_weather)
export(process_weather_month)
export(scale_color_aelab_c)
export(scale_color_aelab_d)
export(scale_colour_aelab_c)
export(scale_colour_aelab_d)
export(scale_fill_aelab_c)
export(scale_fill_aelab_d)
export(sig_labels)
export(tidy_ghg_analyzer)
import(ggplot2)
import(tibble)
importFrom(FSA,dunnTest)
importFrom(dplyr,across)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,where)
importFrom(ggspatial,annotation_north_arrow)
importFrom(ggspatial,annotation_scale)
importFrom(ggspatial,north_arrow_fancy_orienteering)
importFrom(grDevices,colorRampPalette)
importFrom(lubridate,as_datetime)
importFrom(lubridate,ceiling_date)
importFrom(lubridate,days)
importFrom(lubridate,force_tz)
importFrom(lubridate,hours)
importFrom(lubridate,minutes)
importFrom(lubridate,seconds)
importFrom(lubridate,ymd_hms)
importFrom(multcompView,multcompLetters4)
importFrom(openxlsx,convertToDateTime)
importFrom(purrr,map_df)
importFrom(purrr,map_dfr)
importFrom(rcompanion,cldList)
importFrom(readr,read_csv)
importFrom(readxl,read_excel)
importFrom(rlang,"!!")
importFrom(rlang,sym)
importFrom(rnaturalearth,ne_countries)
importFrom(sf,st_as_sf)
importFrom(stats,TukeyHSD)
importFrom(stats,aggregate)
importFrom(stats,aov)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,kruskal.test)
importFrom(stats,lm)
importFrom(stats,na.omit)
importFrom(stats,quantile)
importFrom(stats,shapiro.test)
importFrom(stats,time)
importFrom(stringr,str_detect)
importFrom(stringr,str_remove)
importFrom(stringr,str_replace)
importFrom(tibble,tibble)
importFrom(tidyr,fill)
importFrom(utils,read.csv)

# Overview

**aelab** is an R package for aquatic ecology data analysis. It provides tools for:

1. **GHG flux calculation** — process raw data from LI-COR and LGR gas analyzers, calculate chamber flux, convert units, compute Minimum Detectable Flux (MDF), and aggregate to CO₂-equivalents.
2. **Net ecosystem metabolism (NEM)** — calculate gross primary production (GPP), ecosystem respiration (ER), and net ecosystem production (NEP) from high-frequency dissolved oxygen data.
3. **Statistical analysis** — descriptive statistics, outlier detection, normality testing, data transformation, one-way ANOVA (Tukey HSD), and Kruskal-Wallis (Dunn post-hoc) with compact letter display.
4. **Visualization** — ggplot2-based plot wrappers (point, line, box, bar) with a consistent theme and custom colour palettes.

# Installation

The `aelab` package can be installed from:

1. GitHub (development version):

```r
# install.packages("devtools")
devtools::install_github("Zhao-Jun-Yong/aelab")
```

2. CRAN (stable version):

```r
install.packages("aelab")
```

# Features

For a full walkthrough with worked examples and code output, see the package vignette:

```r
vignette("aelab")
```

#' Processed data from Onset HOBO Dissolved Oxygen Data Logger.
#' A dataset containing 336 dissolved oxygen concentrations changed over time.
#' @format A data.frame with 336 rows and 13 variables:
#' \itemize{
#'   \item date_time: Date and time in POSIXct format.
#'   \item pressure_hpa: Atmospheric pressure (hpa).
#'   \item wind_ms: Wind speed (m/s).
#'   \item do: Dissolved oxygen concentrations (mg/L)
#'   \item temp: Water temperature (Celsius)
#'   \item depth: Water depth (m).
#'   \item salinity: Salinity (ppt).
#'   \item start_date_time: Start date and time of the deployment.
#'   \item end_date_time: End date and time of the deployment.
#'   \item sunrise: Sunrise time during that day.
#'   \item sunset: Sunset time during that day.
#'   \item no_hobo: Name for the data logger .
#'   \item site: Name for the site.
#' }
#' @source own data.
#' @name hobo
NULL

#' @title calc_chla_trichromatic
#' @description Calculate chlorophyll-a concentration from trichromatic
#'   spectrophotometric absorbance readings using the Jeffrey & Humphrey (1975)
#'   equations.
#' @details Absorbance values should be measured in a 1 cm path-length cuvette.
#'   The 750 nm reading is used as a turbidity blank correction.
#'   Formula: \eqn{11.85 \times E_{664} - 1.54 \times E_{647} - 0.08 \times E_{630}}
#'   where \eqn{E_{\lambda} = A_{\lambda} - A_{750}}.
#' @param wl_630 Absorbance at 630 nm.
#' @param wl_647 Absorbance at 647 nm.
#' @param wl_664 Absorbance at 664 nm.
#' @param wl_750 Absorbance at 750 nm (turbidity blank).
#' @return Chlorophyll-a concentration in µg L\eqn{^{-1}} (assuming a 1 cm path
#'   length and standard extraction volume).
#' @examples
#' calc_chla_trichromatic(wl_630 = 0.05, wl_647 = 0.08, wl_664 = 0.20, wl_750 = 0.01)
#' @export
calc_chla_trichromatic <- function(wl_630, wl_647, wl_664, wl_750) {
  e_630 <- wl_630 - wl_750
  e_647 <- wl_647 - wl_750
  e_664 <- wl_664 - wl_750
  (11.85 * e_664) - (1.54 * e_647) - (0.08 * e_630)
}

utils::globalVariables(c("CO2", "CH4", "N2O"))

#' @title tidy_ghg_analyzer
#' @importFrom lubridate ymd_hms days hours minutes seconds force_tz
#' @importFrom readxl read_excel
#' @importFrom openxlsx convertToDateTime
#' @importFrom dplyr filter
#' @description Tidy the data downloaded from GHG Analyzer.
#' @param file_path Directory of file.
#' @param gas Choose between CO2/CH4 or N2O LI-COR Trace Gas Analyzer, which is "ch4" and "n2o", respectively.
#' @param analyzer The brand of the analyzer which the data was downloaded from.
#' @return Return the loaded XLSX file after tidying for further analysis.
#' @examples
#' ghg_data_path <- system.file("extdata", "ch4.xlsx", package = "aelab")
#' tidy_ghg_analyzer(ghg_data_path, "ch4")
#' @export

tidy_ghg_analyzer <- function(file_path, gas, analyzer = "licor") {

  # Import data from the specified Excel file
  data <- readxl::read_excel(file_path)

  # Tidy data based on the type of analyzer
  if(analyzer == "licor") {
    if (gas == "ch4") {
      # Extract column titles for CH4 data
      title <- data[5, c(7:8, 10:11)]
      # Remove unnecessary rows and keep relevant columns
      data <- data[-c(1:6), c(7:8, 10:11)]
      colnames(data) <- title  # Set new column names
      data <- as.data.frame(data)  # Convert to data frame
      data$CH4 <- as.numeric(data$CH4)  # Convert CH4 to numeric
      data$CO2 <- as.numeric(data$CO2)  # Convert CO2 to numeric
      # Filter out rows with NaN values in CH4 and CO2
      data <- dplyr::filter(data, CH4 != "NaN", CO2 != "NaN")
    } else if (gas == "n2o") {
      # Extract column titles for N2O data
      title <- data[5, c(7:8, 10)]
      # Remove unnecessary rows and keep relevant columns
      data <- data[-c(1:6), c(7:8, 10)]
      colnames(data) <- title  # Set new column names
      data <- as.data.frame(data)  # Convert to data frame
      data$N2O <- as.numeric(data$N2O)  # Convert N2O to numeric
      # Filter out rows with NaN values in N2O
      data <- dplyr::filter(data, N2O != "NaN")
    }

    # Convert TIME and DATE columns to date-time format
    data$TIME <- openxlsx::convertToDateTime(data$TIME)
    data$DATE <- openxlsx::convertToDateTime(data$DATE)
    # Format TIME and DATE for readability
    data$TIME <- format(data$TIME, "%H:%M:%S")
    data$DATE <- format(data$DATE, "%Y/%m/%d")
    # Combine DATE and TIME into a single date_time column
    data$date_time <- as.POSIXct(paste(data$DATE, data$TIME), format = "%Y/%m/%d %H:%M:%S")
  } else if(analyzer == "lgr") {
    if (gas == "ch4") {
      # Set column titles for LGR CH4 data
      title <- c("date_time", "CH4", "CO2", "TEMP")
      # Remove unnecessary rows and keep relevant columns
      data <- data[-c(1:2), c(1, 8, 10, 14)]
      colnames(data) <- title  # Set new column names
      data <- as.data.frame(data)  # Convert to data frame
      data$CH4 <- as.numeric(data$CH4)  # Convert CH4 to numeric
      data$CO2 <- as.numeric(data$CO2)  # Convert CO2 to numeric
      # Clean up date_time format by removing milliseconds
      data$date_time <- sub("\\.\\d+", "", data$date_time)
      # Convert date_time to POSIXct format
      data$date_time <- as.POSIXct(data$date_time, format = "%m/%d/%Y %H:%M:%S")
    }
  }

  # Return the tidied data frame
  return(data)

}

#' @title convert_time
#' @description Convert the time of the LI-COR Trace Gas Analyzer to match the time in real life.
#' @param data Data from the LI-COR Trace Gas Analyzer that had been processed by tidy_licor().
#' @param day Day(s) to add or subtract.
#' @param hr Hour(s) to add or subtract.
#' @param min Minute(s) to add or subtract.
#' @param sec Second(s) to add or subtract.
#' @return The input data with a new column in POSIXct format converted based on the input value.
#' @examples
#' data(n2o)
#' converted_n2o <- convert_time(n2o, min = -10, sec = 5)
#' @export

convert_time <- function(data, day = 0, hr = 0, min = 0, sec = 0) {

  # Convert 'date_time' column to POSIXct format
  data$date_time <- lubridate::ymd_hms(data$date_time)

  # Add specified time adjustments to the date_time (lubridate preserves POSIXct directly)
  data$real_datetime <- data$date_time +
    lubridate::days(day) +
    lubridate::hours(hr) +
    lubridate::minutes(min) +
    lubridate::seconds(sec)

  # Return the modified data frame
  return(data)

}

#' @import tibble
#' @importFrom stats lm
#' @importFrom stats coef
#' @importFrom purrr map_dfr
#' @importFrom stringr str_detect
#' @title calculate_regression
#' @description Calculate the slope of greenhouse gas (GHG) concentration change over time using simple linear regression.
#' @param data Data from the GHG analyzer that has been processed and time-converted.
#' @param reference_df A data frame containing measurement reference times, site names, and an \code{analyzer} column.
#' @param ghg Column name in \code{data} containing GHG concentration values (e.g., \code{"CH4"}, \code{"N2O"}).
#' @param reference_time Column name in \code{reference_df} containing the measurement start times (POSIXct).
#' @param site Column name in \code{reference_df} containing site identifiers.
#' @param analyzer_code String pattern used to filter results by the \code{analyzer} column of \code{reference_df}.
#' @param duration_minutes The duration of the measurement, default to 7.
#' @param num_rows The number of rows used to perform the regression, default to 300.
#' @return A tibble containing the time range, slope, R2, site, and analyzer from the simple linear regression.
#' @examples
#' data(n2o)
#' n2o_converted <- convert_time(n2o)
#' ref <- data.frame(
#'   date_time = n2o_converted$real_datetime[1],
#'   site = "S1",
#'   analyzer = "1074"
#' )
#' calculate_regression(n2o_converted, ref, "N2O",
#'                      reference_time = "date_time", site = "site",
#'                      analyzer_code = "1074")
#' @export

calculate_regression <- function(data, reference_df, ghg,
                                 reference_time, site, analyzer_code,
                                 duration_minutes = 7, num_rows = 300) {

  reference_time_vec <- lubridate::force_tz(reference_df[[reference_time]], tzone = "Asia/Taipei")
  reference_site     <- reference_df[[site]]
  reference_analyzer <- reference_df$analyzer

  # Check if 'real_datetime' exists; if not, use 'date_time' as a fallback
  if (!"real_datetime" %in% colnames(data) && "date_time" %in% colnames(data)) {
    data$real_datetime <- data$date_time
  } else {
    # Ensure 'real_datetime' is in the correct timezone
    data$real_datetime <- lubridate::force_tz(data$real_datetime, tzone = "Asia/Taipei")
  }

  # Use map_dfr to iterate through each reference time (avoids O(n^2) rbind)
  results <- purrr::map_dfr(seq_along(reference_time_vec), function(i) {
    reference_datetime <- reference_time_vec[i]

    # Define the start and end time for the regression window
    start_time <- reference_datetime
    end_time <- reference_datetime + (as.numeric(duration_minutes)) * 60

    # Filter data to include only rows within the specified time window
    filtered_data <- data[data$real_datetime >= start_time & data$real_datetime <= end_time, ]

    # Sort the filtered data by 'real_datetime'
    sorted_data <- filtered_data[order(filtered_data$real_datetime), ]

    # Initialize variables to track the best regression results
    best_r_square <- -Inf
    best_slope <- NA
    best_start_time <- NA
    best_end_time <- NA
    cor_site <- NA
    cor_analyzer <- NA

    # Loop through the sorted data to find the best regression fit
    for (j in 1:(nrow(sorted_data) - num_rows + 1)) {
      # Select a subset of data for regression
      selected_data <- sorted_data[j:(j + num_rows - 1), ]

      # Perform linear regression on the selected data
      regression <- stats::lm(as.numeric(selected_data[[ghg]]) ~ seq_along(selected_data[[ghg]]))

      # Calculate R-squared value from the regression summary
      r_square <- summary(regression)$r.squared

      # Update best results if the current R-squared is better
      if (r_square > best_r_square) {
        best_r_square <- r_square
        best_slope <- stats::coef(regression)[2]  # Get the slope from the regression coefficients
        best_start_time <- selected_data$real_datetime[1]  # Record the start time of the best fit
        best_end_time <- selected_data$real_datetime[length(selected_data$real_datetime)]  # Record the end time of the best fit
        cor_site     <- reference_site[i]
        cor_analyzer <- reference_analyzer[i]
      }
    }

    # Return the best results for this reference time
    tibble(
      start_time = format(best_start_time, "%Y/%m/%d %H:%M:%S"),
      end_time = format(best_end_time, "%Y/%m/%d %H:%M:%S"),
      slope = best_slope,
      r_square = best_r_square,
      reference_time = reference_time_vec[i],
      site = cor_site,
      analyzer = cor_analyzer
    )
  })

  # Filter results to the requested analyzer
  results <- results[stringr::str_detect(results$analyzer, analyzer_code), ]

  # Return the results tibble containing regression analysis results
  return(results)

}

#' @title calculate_ghg_flux
#' @description Calculate the greenhouse gas (GHG) flux based on input parameters from a data frame.
#' @param data A data frame containing relevant data with columns for slope, area, volume, and temperature.
#' @param slope Name of the column in `data` that contains the slope values of the GHG concentration change (in ppm/s).
#' @param area Name of the column in `data` that contains the values of the area of the chamber (in square meter).
#' @param volume Name of the column in `data` that contains values of the volume of the chamber (in litre).
#' @param temp Name of the column in `data` that contains values of the temperature of the gas (in Celsius).
#' @return A list containing the calculated flux and its unit.
#' @examples
#' data <- data.frame(
#'   slope = c(1.2, 1.5, 1.1),
#'   area = c(100, 150, 120),
#'   volume = c(10, 15, 12),
#'   temp = c(25, 30, 22)
#' )
#' results <- calculate_ghg_flux(data)
#' print(results)
#' @export

calculate_ghg_flux <- function(data, slope = "slope", area = "area", volume = "volume", temp = "temp") {

  # Constants
  s_to_day <- (1/3600)  # seconds to days
  gas_constant <- 0.082  # gas constant
  celsius_to_kelvin <- 273.15  # Celsius to Kelvin conversion
  micro_to_milli <- 0.001  # micromoles to millimoles conversion

  # Check if specified columns exist in the data frame
  required_cols <- c(slope, area, volume, temp)
  if (!all(required_cols %in% names(data))) {
    stop("One or more specified columns do not exist in the data frame.")
  }

  # Extract values from the data frame
  slope <- data[[slope]]
  area <- data[[area]]
  volume <- data[[volume]]
  temp <- data[[temp]]

  # Calculate flux
  data$flux <- (slope * volume * (1/s_to_day) * micro_to_milli) /
    (gas_constant * (temp + celsius_to_kelvin) * area)

  # Set unit of the result
  data$unit <- "mmol m-2 d-1"

  return(data)
}

#' @title convert_ghg_unit
#' @description Convert a greenhouse gas (GHG) flux value (or a character string
#'   containing one or more numeric values, e.g. \code{"0.002 +/- 0.003"})
#'   to micrograms per square meter per hour.
#' @details Numeric values embedded in a string (e.g. mean +/- SD notation)
#'   are each converted individually and the surrounding text is preserved.
#'   Commas are treated as decimal separators.
#' @param input A single numeric value or a character string containing one or
#'   more numbers.
#' @param ghg The molecular formula of the greenhouse gas: \code{"co2"},
#'   \code{"ch4"}, or \code{"n2o"}.
#' @param mass Mass unit of the input flux. One of \code{"mmol"}, \code{"mg"},
#'   \code{"g"}, \code{"ug"} (micrograms), \code{"nmol"}, \code{"Mg"},
#'   \code{"umol"} (micromoles), \code{"mol"}. Default \code{"ug"}.
#' @param area Area unit of the input flux. One of \code{"ha"}, \code{"m2"}.
#'   Default \code{"m2"}.
#' @param time Time unit of the input flux. One of \code{"yr"}, \code{"day"},
#'   \code{"hr"}, \code{"sec"}, \code{"min"}. Default \code{"hr"}.
#' @param digits Number of decimal places to round to. Default 2.
#' @param ratio Logical. If \code{TRUE}, apply an elemental-ratio correction
#'   (C-basis for CH4, N-basis for N2O). Default \code{FALSE}.
#' @return A named list with \code{value} (converted string) and \code{unit},
#'   or \code{"EMPTY"} for missing/non-numeric input.
#' @examples
#' convert_ghg_unit(97, ghg = "ch4", mass = "mg", area = "m2", time = "hr")
#' @export
convert_ghg_unit <- function(input, ghg, mass = "\u00b5g", area = "m2",
                             time = "hr", digits = 2, ratio = FALSE) {
  if (is.na(input) || input == "") return("EMPTY")

  input <- gsub(",", ".", as.character(input))

  numeric_values <- unlist(regmatches(input,
    gregexpr("[-+]?[0-9]*\\.?[0-9]+", input)))

  if (length(numeric_values) == 0) return("EMPTY")

  valid_ghgs <- c("co2", "ch4", "n2o")
  molar_mass <- switch(ghg,
    "co2" = 44.01, "ch4" = 16.04, "n2o" = 44.01,
    stop(paste("Invalid GHG type. Please use one of:",
               paste(valid_ghgs, collapse = ", "))))

  valid_masses <- c("mmol", "mg", "g", "\u00b5g", "nmol", "Mg", "\u00b5mol", "mol")
  if (!(mass %in% valid_masses))
    stop(paste("Invalid mass unit. Please use one of:",
               paste(valid_masses, collapse = ", ")))

  valid_areas <- c("ha", "m2")
  if (!(area %in% valid_areas))
    stop(paste("Invalid area unit. Please use one of:",
               paste(valid_areas, collapse = ", ")))

  valid_times <- c("yr", "day", "hr", "sec", "min")
  if (!(time %in% valid_times))
    stop(paste("Invalid time unit. Please use one of:",
               paste(valid_times, collapse = ", ")))

  convert_value <- function(value) {
    # Convert mass to µg
    value <- switch(mass,
      "mmol"      = value * molar_mass * 1000,
      "mg"        = value * 1000,
      "g"         = value * 1000000,
      "\u00b5g"   = value,
      "nmol"      = (value * molar_mass) / 1000,
      "Mg"        = value * 1e+12,
      "\u00b5mol" = value * molar_mass,
      "mol"       = value * molar_mass * 1000000
    )
    # Convert area to m2
    if (area == "ha") value <- value / 10000
    # Convert time to hours
    value <- switch(time,
      "yr"  = value / 8760,
      "day" = value / 24,
      "sec" = value * 3600,
      "min" = value * 60,
      "hr"  = value
    )
    # Optional elemental-ratio correction
    if (ratio) {
      if (ghg == "ch4") value <- value * (16.04 / 12.01)
      if (ghg == "n2o") value <- value * (44.013 / 14.0067)
    }
    round(value, digits)
  }

  for (num in numeric_values) {
    converted_num <- convert_value(as.numeric(num))
    input <- gsub(paste0("\\b", num, "\\b"), as.character(converted_num), input)
  }

  list(value = input, unit = "\u00b5g m\u207b\u00b2 h\u207b\u00b9")
}


#' @title calculate_MDF
#' @description Calculate the Minimum Detectable Flux (MDF) for a static chamber
#'   GHG measurement system.
#' @param precision_ppm Precision of the gas analyser (ppm).
#' @param closure_time_s Closure time of the measurement (seconds).
#' @param data_point_n Number of data points recorded during the closure period.
#' @param chamber_volume_m3 Internal volume of the chamber (m\eqn{^3}).
#' @param temperature_C Air temperature at the measurement location (\eqn{^\circ}C).
#' @param chamber_area_m2 Base area of the chamber (m\eqn{^2}).
#' @param pressure_pa Atmospheric pressure (Pa). Default 101325.
#' @param ideal_constant Ideal gas constant (J mol\eqn{^{-1}} K\eqn{^{-1}}).
#'   Default 8.314.
#' @param ghg Greenhouse gas type: \code{"co2"}, \code{"ch4"}, or \code{"n2o"}.
#'   Default \code{"co2"}.
#' @return A named list with \code{MDF} (numeric,
#'   \eqn{\mu}g m\eqn{^{-2}} h\eqn{^{-1}}) and \code{unit} (string).
#' @examples
#' calculate_MDF(
#'   precision_ppm     = 1,
#'   closure_time_s    = 300,
#'   data_point_n      = 300,
#'   chamber_volume_m3 = 0.0064,
#'   temperature_C     = 25,
#'   chamber_area_m2   = 0.07
#' )
#' @export
calculate_MDF <- function(precision_ppm, closure_time_s, data_point_n,
                          chamber_volume_m3, temperature_C, chamber_area_m2,
                          pressure_pa = 101325, ideal_constant = 8.314,
                          ghg = "co2") {
  if (any(c(precision_ppm, closure_time_s, data_point_n,
            chamber_volume_m3, temperature_C, chamber_area_m2) <= 0)) {
    stop("All parameters must be positive and non-zero.")
  }

  molar_masses <- list(co2 = 44.01, ch4 = 16.04, n2o = 44.01)
  if (!(ghg %in% names(molar_masses)))
    stop("Invalid GHG type. Choose from 'co2', 'ch4', or 'n2o'.")

  part1    <- precision_ppm / (closure_time_s * sqrt(data_point_n))
  part2    <- (chamber_volume_m3 * pressure_pa) /
                (ideal_constant * (temperature_C + 273.15) * chamber_area_m2)
  MDF_umol <- part1 * part2 * 3600
  MDF      <- MDF_umol * molar_masses[[ghg]]

  list(MDF = MDF, unit = "\u00b5g m\u207b\u00b2 h\u207b\u00b9")
}


#' @title calculate_total_co2e
#' @description Convert individual GHG fluxes (mg m\eqn{^{-2}} h\eqn{^{-1}})
#'   to a total CO\eqn{_2}-equivalent flux (g m\eqn{^{-2}} d\eqn{^{-1}}) using
#'   IPCC AR6 100-year GWPs (CO\eqn{_2} = 1, CH\eqn{_4} = 27,
#'   N\eqn{_2}O = 273).
#' @param co2 CO\eqn{_2} flux in mg m\eqn{^{-2}} h\eqn{^{-1}}. Default 0.
#' @param ch4 CH\eqn{_4} flux in mg m\eqn{^{-2}} h\eqn{^{-1}}. Default 0.
#' @param n2o N\eqn{_2}O flux in mg m\eqn{^{-2}} h\eqn{^{-1}}. Default 0.
#' @return Total CO\eqn{_2}e flux as a numeric scalar
#'   (g m\eqn{^{-2}} d\eqn{^{-1}}), printed with a diagnostic message.
#' @examples
#' calculate_total_co2e(co2 = 4.02, ch4 = 0.001, n2o = 0.003)
#' @export
calculate_total_co2e <- function(co2 = 0, ch4 = 0, n2o = 0) {
  co2e_mg_hr <- co2 + (ch4 * 27) + (n2o * 273)
  co2e_g_day <- co2e_mg_hr * 0.001 * 24

  message("Calculating total CO2e emissions:")
  message(sprintf("CO2 emissions: %.2f mg m^-2 h^-1", co2))
  message(sprintf("CH4 emissions: %.2f mg m^-2 h^-1 (GWP: 27)", ch4))
  message(sprintf("N2O emissions: %.2f mg m^-2 h^-1 (GWP: 273)", n2o))
  message(sprintf("Total CO2e emissions: %.2f g m^-2 d^-1", co2e_g_day))

  return(co2e_g_day)
}

test_that("tidy_ghg_analyzer works correctly", {
  data <- tidy_ghg_analyzer(test_file("ch4.xlsx"), gas = "ch4")
  expect_true(!is.null(data))
  column_names <- c("DATE", "TIME", "CO2", "CH4", "date_time")
  expect_identical(column_names, names(data))
})

test_that("convert_time adjusts datetime correctly", {
  data <- data.frame(date_time = c("2024/12/16 12:00:00"))
  result <- convert_time(data, hr = 1, min = 30)

  # lubridate::ymd_hms() parses as UTC, so compare with UTC timezone
  expect_equal(result$real_datetime[1], as.POSIXct("2024-12-16 13:30:00", tz = "UTC"))
})

test_that("calculate_regression returns correct results", {
  # Prepare mock data
  data <- data.frame(
    real_datetime = seq.POSIXt(from = as.POSIXct("2024-12-16 12:00:00"),
                               by = "sec", length.out = 420),
    CH4 = sample(1:100, 420, replace = TRUE)
  )

  reference_df <- data.frame(
    date_time = as.POSIXct("2024-12-16 12:00:00", tz = "Asia/Taipei"),
    site = "S1",
    analyzer = "TEST"
  )
  result <- calculate_regression(data, reference_df, "CH4",
                                 reference_time = "date_time",
                                 site = "site",
                                 analyzer_code = "TEST")

  expect_equal(nrow(result), 1)  # Expecting one result
  expect_true("slope" %in% colnames(result))  # Check if slope is calculated
  expect_true("site" %in% colnames(result))
  expect_true("analyzer" %in% colnames(result))
})

test_that("convert_ghg_unit converts correctly", {
  result <- convert_ghg_unit(1, "co2", mass = "mmol")
  expect_equal(as.numeric(result$value), 44010, tolerance = 1)  # 1 mmol co2 = 44.01g/mol * 1000 µg/mg * 1000 = 44010 µg
})

test_that("convert_ghg_unit throws error for invalid GHG", {
  expect_error(convert_ghg_unit(1, "invalid_gas"), "Invalid GHG type")
})

utils::globalVariables(c("long", "lat", "names"))

# ── Internal helper ─────────────────────────────────────────────────────────────

.aelab_theme <- function(style, base_size, text_color) {
  plot_theme <- switch(
    style,
    "bw"        = ggplot2::theme_bw(base_size = base_size),
    "minimal"   = ggplot2::theme_minimal(base_size = base_size),
    "classic"   = ggplot2::theme_classic(base_size = base_size),
    "graycolor" = ggplot2::theme_gray(base_size = base_size),
    "light"     = ggplot2::theme_light(base_size = base_size),
    ggplot2::theme_bw(base_size = base_size)
  )
  plot_theme +
    ggplot2::theme(
      text             = ggplot2::element_text(family = "Century Gothic", color = text_color),
      strip.text       = ggplot2::element_text(face = "bold", color = text_color),
      axis.text        = ggplot2::element_text(face = "bold", color = text_color),
      legend.text      = ggplot2::element_text(face = "bold", color = text_color),
      plot.background  = ggplot2::element_rect(fill = "transparent", color = NA),
      strip.background = ggplot2::element_rect(fill = "transparent", color = NA),
      panel.background = ggplot2::element_rect(fill = "transparent", color = NA),
      legend.background = ggplot2::element_rect(fill = "transparent", color = NA),
      legend.title     = ggplot2::element_blank(),
      legend.position  = "top",
      legend.key       = ggplot2::element_blank()
    )
}

.aelab_facet <- function(plot, facet, facet_x, facet_y) {
  if (!facet) return(plot)
  if (is.null(facet_x)) {
    plot + ggplot2::facet_grid(stats::as.formula(paste(facet_y, "~ .")), scales = "free")
  } else if (is.null(facet_y)) {
    plot + ggplot2::facet_grid(stats::as.formula(paste(". ~", facet_x)), scales = "free")
  } else {
    plot + ggplot2::facet_grid(stats::as.formula(paste(facet_y, "~", facet_x)), scales = "free")
  }
}

# ── Plot functions ───────────────────────────────────────────────────────────────

#' @title plot_point
#' @description Create a scatter plot using the aelab theme.
#' @param df A data frame.
#' @param x <[`data-masking`][ggplot2::aes]> Column mapped to the x-axis.
#' @param y <[`data-masking`][ggplot2::aes]> Column mapped to the y-axis.
#' @param z <[`data-masking`][ggplot2::aes]> Optional column mapped to fill colour.
#' @param base_size Base font size passed to the ggplot2 theme. Default 25.
#' @param point_size Point size. Default 3.
#' @param stroke_size Point stroke width. Default 1.
#' @param text_color Text colour. Default \code{"black"}.
#' @param facet Logical; add facet grid? Default \code{FALSE}.
#' @param facet_x Column name (string) for the horizontal facet dimension.
#' @param facet_y Column name (string) for the vertical facet dimension.
#' @param style Theme style. One of \code{"bw"}, \code{"minimal"},
#'   \code{"classic"}, \code{"graycolor"}, \code{"light"}. Default \code{"bw"}.
#' @return A \code{ggplot} object.
#' @examples
#' \dontrun{
#' df <- data.frame(x = 1:5, y = c(2,4,1,5,3), g = c("A","A","B","B","A"))
#' plot_point(df, x, y, g)
#' }
#' @import ggplot2
#' @importFrom stats as.formula
#' @export
plot_point <- function(df, x, y, z = NULL,
                       base_size = 25, point_size = 3, stroke_size = 1,
                       text_color = "black", facet = FALSE,
                       facet_x = NULL, facet_y = NULL,
                       style = "bw") {
  plot <- ggplot2::ggplot(df, ggplot2::aes(x = {{ x }}, y = {{ y }}, fill = {{ z }})) +
    ggplot2::geom_point(shape = 21, size = point_size, stroke = stroke_size) +
    .aelab_theme(style, base_size, text_color)
  .aelab_facet(plot, facet, facet_x, facet_y)
}


#' @title plot_line
#' @description Create a line plot using the aelab theme.
#' @param df A data frame.
#' @param x <[`data-masking`][ggplot2::aes]> Column mapped to the x-axis.
#' @param y <[`data-masking`][ggplot2::aes]> Column mapped to the y-axis.
#' @param z <[`data-masking`][ggplot2::aes]> Optional column mapped to colour and group.
#' @param base_size Base font size. Default 25.
#' @param line_width Line width. Default 3.
#' @param text_color Text colour. Default \code{"black"}.
#' @param facet Logical; add facet grid? Default \code{FALSE}.
#' @param facet_x Column name (string) for the horizontal facet dimension.
#' @param facet_y Column name (string) for the vertical facet dimension.
#' @param style Theme style. Default \code{"bw"}.
#' @return A \code{ggplot} object.
#' @examples
#' \dontrun{
#' df <- data.frame(x = 1:6, y = c(1,3,2,5,4,6), g = rep(c("A","B"), 3))
#' plot_line(df, x, y, g)
#' }
#' @import ggplot2
#' @importFrom stats as.formula
#' @export
plot_line <- function(df, x, y, z = NULL,
                      base_size = 25, line_width = 3,
                      text_color = "black", facet = FALSE,
                      facet_x = NULL, facet_y = NULL,
                      style = "bw") {
  plot <- ggplot2::ggplot(df, ggplot2::aes(x = {{ x }}, y = {{ y }}, color = {{ z }})) +
    ggplot2::geom_line(ggplot2::aes(group = {{ z }}), linewidth = line_width) +
    .aelab_theme(style, base_size, text_color)
  .aelab_facet(plot, facet, facet_x, facet_y)
}


#' @title plot_box
#' @description Create a box plot with mean overlay using the aelab theme.
#' @param df A data frame.
#' @param x <[`data-masking`][ggplot2::aes]> Column mapped to the x-axis.
#' @param y <[`data-masking`][ggplot2::aes]> Column mapped to the y-axis.
#' @param z <[`data-masking`][ggplot2::aes]> Optional column mapped to fill colour.
#' @param base_size Base font size. Default 25.
#' @param line_width Box outline width. Default 0.5.
#' @param outlier_size Outlier point size. Default 1.5.
#' @param text_color Text colour. Default \code{"black"}.
#' @param facet Logical; add facet grid? Default \code{FALSE}.
#' @param facet_x Column name (string) for the horizontal facet dimension.
#' @param facet_y Column name (string) for the vertical facet dimension.
#' @param style Theme style. Default \code{"bw"}.
#' @return A \code{ggplot} object.
#' @examples
#' \dontrun{
#' df <- data.frame(x = rep(c("A","B"), each = 5), y = c(1:5, 3:7))
#' plot_box(df, x, y)
#' }
#' @import ggplot2
#' @importFrom stats as.formula
#' @export
plot_box <- function(df, x, y, z = NULL,
                     base_size = 25, line_width = 0.5,
                     outlier_size = 1.5,
                     text_color = "black", facet = FALSE,
                     facet_x = NULL, facet_y = NULL,
                     style = "bw") {
  plot <- ggplot2::ggplot(df, ggplot2::aes(x = {{ x }}, y = {{ y }}, fill = {{ z }})) +
    ggplot2::geom_boxplot(linewidth = line_width, outlier.size = outlier_size) +
    .aelab_theme(style, base_size, text_color) +
    ggplot2::stat_summary(
      ggplot2::aes(x = {{ x }}, y = {{ y }}, fill = {{ z }}),
      fun = mean, geom = "point", shape = 22,
      size = 2, color = "black",
      stroke = line_width,
      position = ggplot2::position_dodge(width = 0.75)
    ) +
    ggplot2::theme(legend.key = NULL)
  .aelab_facet(plot, facet, facet_x, facet_y)
}


#' @title plot_bar
#' @description Create a bar plot using the aelab theme.
#' @param df A data frame.
#' @param x <[`data-masking`][ggplot2::aes]> Column mapped to the x-axis.
#' @param y <[`data-masking`][ggplot2::aes]> Column mapped to the y-axis.
#' @param z <[`data-masking`][ggplot2::aes]> Optional column mapped to fill colour.
#' @param base_size Base font size. Default 25.
#' @param line_width Bar outline width. Default 1.
#' @param text_color Text colour. Default \code{"black"}.
#' @param facet Logical; add facet grid? Default \code{FALSE}.
#' @param facet_x Column name (string) for the horizontal facet dimension.
#' @param facet_y Column name (string) for the vertical facet dimension.
#' @param style Theme style. Default \code{"bw"}.
#' @param position Bar position: \code{"dodge"} or \code{"stack"}. Default \code{"dodge"}.
#' @param stat Stat type: \code{"identity"} or \code{"count"}. Default \code{"identity"}.
#' @return A \code{ggplot} object.
#' @examples
#' \dontrun{
#' df <- data.frame(x = c("A","B","A","B"), g = c("X","X","Y","Y"), y = c(1,2,3,4))
#' plot_bar(df, x, y, g)
#' }
#' @import ggplot2
#' @importFrom stats as.formula
#' @export
plot_bar <- function(df, x, y, z = NULL,
                     base_size = 25, line_width = 1,
                     text_color = "black", facet = FALSE,
                     facet_x = NULL, facet_y = NULL,
                     style = "bw", position = "dodge",
                     stat = "identity") {
  plot <- ggplot2::ggplot(df, ggplot2::aes(x = {{ x }}, y = {{ y }}, fill = {{ z }})) +
    ggplot2::geom_bar(ggplot2::aes(fill = {{ z }}),
                      position = position, stat = stat,
                      linewidth = line_width, color = "black") +
    .aelab_theme(style, base_size, text_color)
  .aelab_facet(plot, facet, facet_x, facet_y)
}

# ── Palette system ───────────────────────────────────────────────────────────────

#' @title aelab_palettes
#' @description Retrieve a named aelab colour palette as a character vector.
#' @details Available palette names: \code{"rainbow"}, \code{"two"},
#'   \code{"control"}, \code{"control2"}, \code{"control3"},
#'   \code{"period"}, \code{"ghg"}.
#' @param name Name of the palette (string).
#' @param n Number of colours to return. Defaults to the full palette length.
#' @param type \code{"discrete"} (default) or \code{"continuous"}. For
#'   \code{"continuous"}, colours are interpolated to length \code{n}.
#' @return A character vector of hex colour codes with class \code{"palette"}.
#' @examples
#' aelab_palettes("rainbow", 5)
#' aelab_palettes("ghg", type = "continuous", n = 20)
#' @importFrom grDevices colorRampPalette
#' @export
aelab_palettes <- function(name, n, type = c("discrete", "continuous")) {
  aelab_colours <- list(
    rainbow  = c("#C98686", "#f0a082", "#F2B880", "#eddea4",
                 "#8fcb9b", "#5b9279", "#9db4c0", "#71869a",
                 "#b8a1b4", "#eae6e5", "#a8a59a"),
    two      = c("#9db4c0", "#eddea4"),
    control  = c("#adb5bd", "#F2B880", "#f0a082", "#C98686"),
    control2 = c("#adb5bd", "#6c757d", "#F2B880", "#f0a082"),
    control3 = c("#adb5bd", "#F2B880", "#6c757d", "#f0a082"),
    period   = c("#adb5bd", "#e9c46a", "#6c757d"),
    ghg      = c("#e9c46a", "#adb5bd", "#6c757d")
  )

  palette <- aelab_colours[[name]]
  if (is.null(palette)) stop(paste("Unknown palette:", name))
  if (missing(n)) n <- length(palette)
  type <- match.arg(type)

  out <- switch(type,
    continuous = grDevices::colorRampPalette(palette)(n),
    discrete   = palette[seq_len(n)]
  )
  structure(out, name = name, class = "palette")
}


#' @title scale_colour_aelab_d
#' @description Discrete ggplot2 colour scale using an aelab palette.
#' @param name Palette name passed to \code{\link{aelab_palettes}}.
#' @param direction \code{1} (default) for normal order; \code{-1} to reverse.
#' @return A \code{ggplot2} scale.
#' @examples
#' \dontrun{
#' ggplot2::ggplot(mtcars, ggplot2::aes(wt, mpg, colour = factor(cyl))) +
#'   ggplot2::geom_point() + scale_colour_aelab_d("rainbow")
#' }
#' @import ggplot2
#' @export
scale_colour_aelab_d <- function(name, direction = 1) {
  values <- aelab_palettes(name, type = "discrete")
  if (direction == -1) values <- rev(values)
  ggplot2::scale_colour_manual(values = values)
}

#' @rdname scale_colour_aelab_d
#' @export
scale_color_aelab_d <- scale_colour_aelab_d


#' @title scale_fill_aelab_d
#' @description Discrete ggplot2 fill scale using an aelab palette.
#' @param name Palette name passed to \code{\link{aelab_palettes}}.
#' @param direction \code{1} (default) for normal order; \code{-1} to reverse.
#' @return A \code{ggplot2} scale.
#' @examples
#' \dontrun{
#' ggplot2::ggplot(mtcars, ggplot2::aes(factor(cyl), fill = factor(cyl))) +
#'   ggplot2::geom_bar() + scale_fill_aelab_d("control")
#' }
#' @import ggplot2
#' @export
scale_fill_aelab_d <- function(name, direction = 1) {
  values <- aelab_palettes(name, type = "discrete")
  if (direction == -1) values <- rev(values)
  ggplot2::scale_fill_manual(values = values)
}


#' @title scale_colour_aelab_c
#' @description Continuous ggplot2 colour scale using an aelab palette.
#' @param name Palette name passed to \code{\link{aelab_palettes}}.
#' @param direction \code{1} (default) for normal order; \code{-1} to reverse.
#' @return A \code{ggplot2} scale.
#' @examples
#' \dontrun{
#' ggplot2::ggplot(mtcars, ggplot2::aes(wt, mpg, colour = mpg)) +
#'   ggplot2::geom_point() + scale_colour_aelab_c("rainbow")
#' }
#' @import ggplot2
#' @export
scale_colour_aelab_c <- function(name, direction = 1) {
  values <- aelab_palettes(name, type = "continuous")
  if (direction == -1) values <- rev(values)
  ggplot2::scale_colour_manual(values = values)
}

#' @rdname scale_colour_aelab_c
#' @export
scale_color_aelab_c <- scale_colour_aelab_c


#' @title scale_fill_aelab_c
#' @description Continuous ggplot2 fill scale using an aelab palette.
#' @param name Palette name passed to \code{\link{aelab_palettes}}.
#' @param direction \code{1} (default) for normal order; \code{-1} to reverse.
#' @return A \code{ggplot2} scale.
#' @examples
#' \dontrun{
#' ggplot2::ggplot(mtcars, ggplot2::aes(factor(cyl), fill = mpg)) +
#'   ggplot2::geom_col() + scale_fill_aelab_c("ghg")
#' }
#' @import ggplot2
#' @export
scale_fill_aelab_c <- function(name, direction = 1) {
  values <- aelab_palettes(name, type = "continuous")
  if (direction == -1) values <- rev(values)
  ggplot2::scale_fill_manual(values = values)
}

# ── Map ──────────────────────────────────────────────────────────────────────────

#' @title plot_map_taiwan
#' @description Plot sampling sites on a map of Taiwan with a north arrow and
#'   scale bar.
#' @param long Numeric vector of longitudes.
#' @param lat Numeric vector of latitudes.
#' @param names Character vector of site labels (same length as \code{long}).
#' @param color Fill colour for site markers. Default \code{"darkgrey"}.
#' @param textsize Size for annotation and point labels. Default 5.
#' @param basesize Base font size for the map theme. Default 16.
#' @param shape_type ggplot2 point shape number. Default 22 (filled square).
#' @return A \code{ggplot} object (also printed to the active device).
#' @examples
#' \dontrun{
#' plot_map_taiwan(
#'   long = c(120.2, 121.5),
#'   lat  = c(22.9, 24.1),
#'   names = c("Site A", "Site B")
#' )
#' }
#' @import ggplot2
#' @importFrom rnaturalearth ne_countries
#' @importFrom sf st_as_sf
#' @importFrom ggspatial annotation_north_arrow north_arrow_fancy_orienteering
#'   annotation_scale
#' @export
plot_map_taiwan <- function(long, lat, names, color = "darkgrey",
                            textsize = 5, basesize = 16, shape_type = 22) {
  if (!is.numeric(long) || !is.numeric(lat) || length(long) != length(lat)) {
    stop("Invalid latitude or longitude provided.")
  }

  world <- rnaturalearth::ne_countries(scale = "medium", returnclass = "sf")

  map <- ggplot2::ggplot(data = world) +
    ggplot2::geom_sf() +
    ggplot2::xlab(expression(bold("Longitude"))) +
    ggplot2::ylab(expression(bold("Latitude"))) +
    ggplot2::coord_sf(xlim = c(118.50, 123.50), ylim = c(21.00, 26.00),
                      expand = FALSE) +
    ggplot2::theme_bw(base_size = basesize)

  for (i in seq_along(long)) {
    site_df <- data.frame(long = long[i], lat = lat[i], names = names[i])
    map <- map +
      ggplot2::geom_point(data = site_df,
                          ggplot2::aes(x = long, y = lat),
                          size = textsize, shape = shape_type,
                          color = "black", fill = color, stroke = 1) +
      ggplot2::annotate(geom = "text", x = long[i], y = lat[i] + 0.2,
                        label = names[i],
                        fontface = "bold", size = textsize - 1, colour = "black")
  }

  map <- map +
    ggplot2::annotate(geom = "text", x = 121, y = 24, label = "Taiwan",
                      fontface = "italic", size = textsize) +
    ggspatial::annotation_north_arrow(
      location   = "bl", which_north = "true",
      pad_x      = ggplot2::unit(0.75, "in"),
      pad_y      = ggplot2::unit(0.5, "in"),
      style      = ggspatial::north_arrow_fancy_orienteering
    ) +
    ggspatial::annotation_scale(location = "bl", width_hint = 0.5)

  print(map)
}

#' Processed data from N2O LI-COR Trace Gas Analyzer.
#' A dataset containing 567 N2O concentrations changed over time.
#' @format A data.frame with 567 rows and 4 variables:
#' \itemize{
#'   \item DATE: Date in character format.
#'   \item TIME: Time in character format.
#'   \item N2O: Concentrations of nitrous oxide (N2O), in ppb.
#'   \item date_time: Date and time in POSIXct format.
#' }
#' @source own data.
#' @name n2o
NULL

utils::globalVariables(c("cur_column"))

#' @title descriptive_statistic
#' @description Compute grouped mean ± SD and min–max summary statistics for
#'   one or more numeric variables.
#' @param data A data frame.
#' @param vars <[`tidy-select`][dplyr::dplyr_tidy_select]> Columns to summarise.
#' @param groups <[`tidy-select`][dplyr::dplyr_tidy_select]> Grouping columns.
#' @param digits Number of decimal places to round to. Default is 2.
#' @return A tibble with one row per group and two summary columns per variable
#'   (`<var>_mean_sd` and `<var>_min_max`).
#' @examples
#' df <- data.frame(group = c("A","A","B","B"), value = c(1.1, 2.3, 3.5, 4.7))
#' descriptive_statistic(df, vars = value, groups = group)
#' @importFrom dplyr group_by summarise across
#' @export
descriptive_statistic <- function(data, vars, groups, digits = 2) {
  data |>
    dplyr::group_by(dplyr::across({{ groups }}, as.character)) |>
    dplyr::summarise(
      dplyr::across(
        {{ vars }},
        .fns = list(
          mean_sd = ~ paste(
            format(round(mean(., na.rm = TRUE), digits), scientific = FALSE),
            format(round(sd(., na.rm = TRUE), digits), scientific = FALSE),
            sep = " \u00B1 "
          ),
          min_max = ~ paste(
            format(round(min(., na.rm = TRUE), digits), scientific = FALSE),
            format(round(max(., na.rm = TRUE), digits), scientific = FALSE),
            sep = " to "
          )
        )
      ),
      .groups = "drop"
    )
}


#' @title normality_test_t
#' @description Test normality of a variable within two groups using Shapiro-Wilk
#'   on raw, square-root, and log10 transforms (for t-test context).
#' @param df A data frame.
#' @param variable_name Name of the numeric variable column (string).
#' @param group <[`data-masking`][dplyr::dplyr_data_masking]> The grouping column.
#' @param group_1 Value identifying the first group.
#' @param group_2 Value identifying the second group.
#' @return A tibble with Shapiro-Wilk p-values for each group × transformation
#'   combination.
#' @examples
#' df <- data.frame(
#'   grp = c("A","A","A","B","B","B"),
#'   val = c(1.1, 2.0, 1.5, 4.2, 3.8, 4.5)
#' )
#' normality_test_t(df, "val", grp, "A", "B")
#' @importFrom dplyr filter
#' @importFrom tibble tibble
#' @importFrom stats shapiro.test
#' @export
normality_test_t <- function(df, variable_name, group, group_1, group_2) {
  df <- df[!is.na(df[[variable_name]]), ]
  variable <- df[[variable_name]]
  max_value <- max(variable) + 1

  df_1 <- dplyr::filter(df, {{ group }} == group_1)
  df_2 <- dplyr::filter(df, {{ group }} == group_2)

  test_1      <- stats::shapiro.test(df_1[[variable_name]])
  test_2      <- stats::shapiro.test(df_2[[variable_name]])
  test_sqrt_1 <- stats::shapiro.test(sqrt(max_value - df_1[[variable_name]]))
  test_sqrt_2 <- stats::shapiro.test(sqrt(max_value - df_2[[variable_name]]))
  test_log_1  <- stats::shapiro.test(log10(max_value - df_1[[variable_name]]))
  test_log_2  <- stats::shapiro.test(log10(max_value - df_2[[variable_name]]))

  tibble::tibble(
    variable       = variable_name,
    group          = c(group_1, group_2, group_1, group_2, group_1, group_2),
    p_value        = c(test_1$p.value, test_2$p.value,
                       test_sqrt_1$p.value, test_sqrt_2$p.value,
                       test_log_1$p.value, test_log_2$p.value),
    transformation = c("None", "None",
                       "Square root", "Square root",
                       "Logarithm", "Logarithm")
  )
}


#' @title normality_test_aov
#' @description Test normality of ANOVA model residuals using Shapiro-Wilk on
#'   raw, square-root, and log10 transforms (one-way or two-way).
#' @param df A data frame.
#' @param variable_name Name of the response variable column (string).
#' @param group_1 Name of the first grouping column (string).
#' @param group_2 Name of the second grouping column (string), or \code{NULL}
#'   for a one-way model.
#' @return A tibble with Shapiro-Wilk p-values for each transformation.
#' @examples
#' df <- data.frame(
#'   grp = c("A","A","B","B"),
#'   val = c(1.1, 1.4, 3.2, 3.8)
#' )
#' normality_test_aov(df, "val", "grp")
#' @importFrom tibble tibble
#' @importFrom stats lm shapiro.test
#' @export
normality_test_aov <- function(df, variable_name, group_1, group_2 = NULL) {
  df       <- as.data.frame(df)
  df       <- df[!is.na(df[[variable_name]]), ]
  variable <- df[[variable_name]]
  max_value <- max(variable) + 1

  if (is.null(group_2)) {
    model1 <- stats::lm(variable ~ df[[group_1]], data = df)
    test1  <- stats::shapiro.test(model1$residuals)
    model2 <- stats::lm(sqrt(max_value - variable) ~ df[[group_1]], data = df)
    test2  <- stats::shapiro.test(model2$residuals)
    model3 <- stats::lm(log10(max_value - variable) ~ df[[group_1]], data = df)
    test3  <- stats::shapiro.test(model3$residuals)
  } else {
    model1 <- stats::lm(variable ~ df[[group_1]] * df[[group_2]], data = df)
    test1  <- stats::shapiro.test(model1$residuals)
    model2 <- stats::lm(sqrt(max_value - variable) ~ df[[group_1]] * df[[group_2]], data = df)
    test2  <- stats::shapiro.test(model2$residuals)
    model3 <- stats::lm(log10(max_value - variable) ~ df[[group_1]] * df[[group_2]], data = df)
    test3  <- stats::shapiro.test(model3$residuals)
  }

  tibble::tibble(
    variable       = variable_name,
    p_value        = c(test1$p.value, test2$p.value, test3$p.value),
    transformation = c("None", "Square root", "Logarithm")
  )
}


#' @title aov_test
#' @description Perform one-way ANOVA followed by Tukey HSD post-hoc test with
#'   compact letter display.
#' @param df A data frame.
#' @param variable_name Name of the response variable column (string).
#' @param group Name of the grouping column (string).
#' @return A named list with elements \code{anova_summary}, \code{tukey_results},
#'   and \code{compact_letters}.
#' @examples
#' df <- data.frame(
#'   grp = rep(c("A","B","C"), each = 5),
#'   val = c(1,2,1,2,1, 3,4,3,4,3, 5,6,5,6,5)
#' )
#' aov_test(df, "val", "grp")
#' @importFrom stats aov TukeyHSD as.formula
#' @importFrom multcompView multcompLetters4
#' @export
aov_test <- function(df, variable_name, group) {
  formula        <- stats::as.formula(paste(variable_name, "~", group))
  results        <- stats::aov(formula, data = df)
  anova_summary  <- summary(results)
  print(anova_summary)
  tukey_results  <- stats::TukeyHSD(results)
  label          <- multcompView::multcompLetters4(results, tukey_results)
  print(label)

  list(
    anova_summary   = anova_summary,
    tukey_results   = tukey_results,
    compact_letters = label
  )
}


#' @title ks_test
#' @description Perform Kruskal-Wallis test followed by Dunn post-hoc test
#'   (Bonferroni correction) with compact letter display.
#' @param df A data frame.
#' @param variable_name Name of the response variable column (string).
#' @param group Name of the grouping column (string).
#' @return A named list with elements \code{ks_results}, \code{dunn_results},
#'   \code{mean_summary}, and \code{compact_letters}.
#' @examples
#' df <- data.frame(
#'   grp = rep(c("A","B","C"), each = 5),
#'   val = c(1,2,1,2,1, 3,4,3,4,3, 5,6,5,6,5)
#' )
#' ks_test(df, "val", "grp")
#' @importFrom stats kruskal.test as.formula
#' @importFrom dplyr group_by summarise
#' @importFrom rlang sym !!
#' @importFrom FSA dunnTest
#' @importFrom rcompanion cldList
#' @export
ks_test <- function(df, variable_name, group) {
  if (!variable_name %in% names(df)) {
    stop(paste("Variable", variable_name, "not found in dataframe."))
  }
  if (!group %in% names(df)) {
    stop(paste("Group", group, "not found in dataframe."))
  }

  formula    <- stats::as.formula(paste(variable_name, "~", group))
  ks_results <- stats::kruskal.test(formula, data = df)

  # cldList -> multcompLetters -> vec2mat2 splits comparison names on "-",
  # expecting exactly one per name. Sanitize group names containing "-" first.
  orig_levels <- as.character(unique(df[[group]]))
  safe_levels <- gsub("-", ".", orig_levels, fixed = TRUE)
  level_map   <- stats::setNames(orig_levels, safe_levels)  # safe -> orig

  df_safe          <- df
  df_safe[[group]] <- gsub("-", ".", as.character(df[[group]]), fixed = TRUE)

  dunn_results <- FSA::dunnTest(df_safe[[variable_name]], df_safe[[group]], method = "bonferroni")
  dunn_results <- dunn_results$res

  label <- rcompanion::cldList(
    comparison = dunn_results$Comparison,
    p.value    = dunn_results$P.adj,
    threshold  = 0.05
  )

  # Restore original group names (replace longest safe names first to avoid
  # partial-match corruption, e.g. "CR.I" vs "CR.I1")
  sorted_safe <- names(level_map)[order(nchar(names(level_map)), decreasing = TRUE)]
  label$Group <- level_map[label$Group]
  dunn_results$Comparison <- Reduce(
    function(s, safe) gsub(safe, level_map[[safe]], s, fixed = TRUE),
    sorted_safe,
    dunn_results$Comparison
  )

  mean_summary <- df |>
    dplyr::group_by(!!rlang::sym(group)) |>
    dplyr::summarise(mean = mean(!!rlang::sym(variable_name), na.rm = TRUE),
                     .groups = "drop")

  list(
    ks_results      = ks_results,
    dunn_results    = dunn_results,
    mean_summary    = mean_summary,
    compact_letters = label
  )
}



#' @title sig_labels
#' @description Run \code{\link{ks_test}} separately for each level of a
#'   faceting variable and return compact letter display (CLD) annotations
#'   ready for \code{geom_text}. Only facets where the Kruskal-Wallis p-value
#'   is below \code{alpha} are included in the output.
#' @param stat_data Data frame of raw observations used for the statistical
#'   test. Should already be filtered to the relevant subset (e.g. a single
#'   water type).
#' @param variable Name of the response variable column (string).
#' @param group Name of the grouping column (string).
#' @param by Name of the faceting column whose levels are iterated over
#'   (string, default \code{"year"}).
#' @param plot_data Data frame used to compute label y-positions via
#'   \code{max(variable)} per group. Defaults to \code{stat_data}. Pass the
#'   aggregated/boxplot data here when it differs from the raw test data.
#' @param alpha Significance threshold; facets with KW p-value >=
#'   \code{alpha} are dropped (default \code{0.05}).
#' @return A data frame with columns \code{<group>}, \code{Letter},
#'   \code{MonoLetter}, \code{y_pos}, and \code{<by>}. Returns an empty data
#'   frame when no facet reaches significance.
#' @examples
#' set.seed(1)
#' df <- data.frame(
#'   year = rep(c("2023", "2024"), each = 20),
#'   grp  = rep(c("A","B","C","D"), 10),
#'   val  = c(rnorm(20, mean = rep(c(1,3,2,6), 5)), rnorm(20))
#' )
#' sig_labels(df, "val", "grp", by = "year")
#' @importFrom purrr map_df
#' @export
sig_labels <- function(stat_data, variable, group, by = "year",
                       plot_data = NULL, alpha = 0.05) {
  if (is.null(plot_data)) plot_data <- stat_data
  purrr::map_df(as.character(unique(stat_data[[by]])), function(val) {
    d_s <- stat_data[as.character(stat_data[[by]]) == val, ]
    d_p <- plot_data[as.character(plot_data[[by]]) == val, ]
    if (length(unique(d_s[[group]])) < 2) return(data.frame())
    res <- tryCatch(ks_test(d_s, variable, group), error = function(e) NULL)
    if (is.null(res) || res$ks_results$p.value >= alpha) return(data.frame())
    lbl           <- res$compact_letters
    names(lbl)[names(lbl) == "Group"] <- group
    ypos          <- tapply(d_p[[variable]], d_p[[group]], max, na.rm = TRUE)
    lbl$y_pos     <- ypos[as.character(lbl[[group]])]
    lbl[[by]]     <- val
    lbl
  })
}


#' @title df_trans
#' @description Apply a reverse square-root or reverse log transformation to a
#'   numeric column and append the result as a new column.
#' @param df A data frame.
#' @param variable_name Name of the column to transform (string).
#' @param transformation Transformation type: \code{"sqrt"} or \code{"log"}.
#' @return The input data frame with an additional column named
#'   \code{<variable_name>_sqrt} or \code{<variable_name>_log}.
#' @examples
#' df <- data.frame(val = c(1, 4, 9, 16))
#' df_trans(df, "val", "sqrt")
#' @export
df_trans <- function(df, variable_name, transformation) {
  max_val <- max(df[[variable_name]], na.rm = TRUE)
  if (transformation == "sqrt") {
    df[[paste0(variable_name, "_sqrt")]] <- sqrt((max_val + 1) - df[[variable_name]])
  } else if (transformation == "log") {
    df[[paste0(variable_name, "_log")]]  <- log((max_val + 1) - df[[variable_name]])
  }
  return(df)
}


#' @title find_outlier
#' @description Identify outliers in a numeric column using the IQR method
#'   (values outside 1.5 \eqn{\times} IQR from Q1/Q3).
#' @param df A data frame.
#' @param var Name of the column to check for outliers (string).
#' @param other_var Character vector of additional column names to return
#'   alongside the outlier values, or \code{NULL}.
#' @return A tibble with columns \code{row_index}, \code{outlier_value}, and any
#'   requested \code{other_var} columns.
#' @examples
#' df <- data.frame(val = c(1, 2, 2, 3, 100), id = 1:5)
#' find_outlier(df, "val", "id")
#' @importFrom tibble tibble
#' @importFrom stats quantile
#' @export
find_outlier <- function(df, var, other_var = NULL) {
  Q3  <- stats::quantile(df[[var]], 0.75, na.rm = TRUE)
  Q1  <- stats::quantile(df[[var]], 0.25, na.rm = TRUE)
  IQR <- Q3 - Q1
  outlier_indices <- which(df[[var]] < (Q1 - 1.5 * IQR) | df[[var]] > (Q3 + 1.5 * IQR))
  outlier_values  <- df[outlier_indices, var]

  if (!is.null(other_var)) {
    additional_values <- df[outlier_indices, other_var, drop = FALSE]
    tibble::tibble(row_index = outlier_indices, outlier_value = outlier_values,
                   additional_values)
  } else {
    tibble::tibble(row_index = outlier_indices, outlier_value = outlier_values)
  }
}

test_that("ks_test returns correct structure", {
  df <- data.frame(
    grp = rep(c("A", "B", "C"), each = 5),
    val = c(1, 2, 1, 2, 1, 3, 4, 3, 4, 3, 5, 6, 5, 6, 5)
  )
  res <- ks_test(df, "val", "grp")
  expect_named(res, c("ks_results", "dunn_results", "mean_summary", "compact_letters"))
  expect_s3_class(res$ks_results, "htest")
  expect_true(all(c("Group", "Letter") %in% names(res$compact_letters)))
})

test_that("ks_test handles group names containing hyphens without error", {
  df <- data.frame(
    grp = rep(c("CR-I", "CR-II", "CR-III"), each = 5),
    val = c(1, 2, 1, 2, 1, 3, 4, 3, 4, 3, 5, 6, 5, 6, 5)
  )
  res <- ks_test(df, "val", "grp")
  expect_true(all(res$compact_letters$Group %in% c("CR-I", "CR-II", "CR-III")))
  expect_true(all(grepl("-", res$dunn_results$Comparison)))
})

test_that("sig_labels returns non-empty data frame for data with clear group differences", {
  df <- data.frame(
    year = rep("2023", 20),
    grp  = rep(c("A", "B", "C", "D"), each = 5),
    val  = c(rep(1, 5), rep(10, 5), rep(1, 5), rep(10, 5))
  )
  result <- sig_labels(df, "val", "grp", by = "year")
  expect_gt(nrow(result), 0)
})

test_that("sig_labels returns empty data frame when no significant difference", {
  set.seed(1)
  df <- data.frame(
    year = rep("2023", 20),
    grp  = rep(c("A", "B", "C", "D"), 5),
    val  = rnorm(20)
  )
  result <- sig_labels(df, "val", "grp", by = "year")
  expect_equal(nrow(result), 0)
})

test_that("sig_labels output contains the by column", {
  set.seed(1)
  df <- data.frame(
    year = rep(c("2023", "2024"), each = 20),
    grp  = rep(c("A", "B", "C", "D"), 10),
    val  = c(rnorm(20, mean = rep(c(1, 3, 2, 6), 5)), rnorm(20))
  )
  result <- sig_labels(df, "val", "grp", by = "year")
  expect_true("year" %in% names(result))
})

test_that("plot_data overrides y_pos relative to using stat_data alone", {
  set.seed(1)
  df <- data.frame(
    year = rep("2023", 20),
    grp  = rep(c("A", "B", "C", "D"), 5),
    val  = rnorm(20, mean = rep(c(1, 3, 2, 6), 5))
  )
  df_inflated <- df
  df_inflated$val <- df$val + 100

  res_default  <- sig_labels(df, "val", "grp", by = "year")
  res_plotdata <- sig_labels(df, "val", "grp", by = "year", plot_data = df_inflated)

  expect_false(identical(res_default$y_pos, res_plotdata$y_pos))
})

utils::globalVariables(c(
  "sunset", "sunrise", "date_time", "temp", "temp_k", "salinity",
  "c_o2", "o2_saturation", "pressure_hpa", "wind_ms", "k600", "sc",
  "cor_o2_saturation_pressure", "k", "rate_do_change", "depth_m",
  "flux", "time", "sunrise_time", "sunset_time", "site", "no_hobo",
  "nep_hr", "daylight_hr", "r_hr", "nep_daytime", "r_daytime",
  "gpp", "r_day", "nep", "hobo", "do", "date"
))



#' @title process_hobo
#' @importFrom lubridate ceiling_date hours days force_tz
#' @importFrom dplyr arrange
#' @importFrom utils read.csv
#' @importFrom stats na.omit
#' @importFrom stats aggregate
#' @importFrom stats time
#' @description Tidy the data retrieved from HOBO U26 Dissolved Oxygen Data Logger.
#' @param file_path Directory of file.
#' @param no_hobo The code for the data logger.
#' @return A dataframe.
#' @examples
#' hobo_data_path <- system.file("extdata", "ex_hobo.csv", package = "aelab")
#' df <- process_hobo(hobo_data_path, "code_for_logger")
#' @export

process_hobo <- function(file_path, no_hobo) {
  # Read the CSV file without headers
  df <- utils::read.csv(file_path, header = F)

  # Remove the first two rows and columns 5 to 9
  df <- df[-c(1:2), -c(5:9)]

  # Set column names for the dataframe
  colnames(df) <- c("no", "date_time", "do", "temp")

  # Convert to a data frame and remove rows with NA values
  df <- as.data.frame(df)
  df <- stats::na.omit(df)

  # Check if date_time contains "上午" (morning) or "下午" (afternoon)
  if (any(grepl("\u4e0a\u5348|\u4e0b\u5348", df$date_time))) {

    # Create a new variable to indicate morning or afternoon
    df$new_variable <- ifelse(grepl("\u4e0a\u5348", df$date_time), "morning", "afternoon")

    # Clean up the date_time string by removing unnecessary characters
    df$date_time <- gsub("\u4e0a\u5348|\u4e0b\u5348", "", df$date_time)  # Remove "上午" and "下午"
    df$date_time <- gsub("\u6642", ":", df$date_time)                     # Replace "時" with ":"
    df$date_time <- gsub("\u5206", ":", df$date_time)                     # Replace "分" with ":"
    df$date_time <- gsub("\u79d2", "", df$date_time)                      # Remove "秒"

    # Convert the cleaned date_time to POSIXlt format
    df$date_time <- strptime(df$date_time, format = "%m/%d/%Y %H:%M:%S")

    # Create subsets for afternoon and morning based on time conditions
    subset_afternoon <- df[df$new_variable == "afternoon" &
                             format(df$date_time, "%H:%M:%S") >= "01:00:00" &
                             format(df$date_time, "%H:%M:%S") <= "11:59:59", ]
    subset_morning <- df[df$new_variable == "morning" &
                           format(df$date_time, "%H:%M:%S") >= "12:00:00" &
                           format(df$date_time, "%H:%M:%S") <= "12:59:59", ]

    # Adjust the date_time for afternoon and morning subsets
    subset_afternoon$date_time <- subset_afternoon$date_time + lubridate::hours(12)  # Convert to 24-hour format
    subset_morning$date_time <- subset_morning$date_time + lubridate::hours(12) - lubridate::days(1)  # Adjust for previous day

    # Update the original dataframe with the adjusted subsets
    df[df$new_variable == "afternoon" &
         format(df$date_time, "%H:%M:%S") >= "01:00:00" &
         format(df$date_time, "%H:%M:%S") <= "11:59:59", ] <- subset_afternoon
    df[df$new_variable == "morning" &
         format(df$date_time, "%H:%M:%S") >= "12:00:00" &
         format(df$date_time, "%H:%M:%S") <= "12:59:59", ] <- subset_morning

    # Clean up temporary variables
    rm(subset_afternoon, subset_morning)
    df$new_variable <- NULL  # Remove the new_variable column

  } else {
    # No action needed if date_time does not contain morning or afternoon indicators
  }

  # Convert 'do' and 'temp' columns to numeric
  df$do <- as.numeric(df$do)
  df$temp <- as.numeric(df$temp)

  # Extract time from date_time for further analysis
  df$time <- format(df$date_time, "%H:%M:%S")

  # Round date_time to the nearest 30 minutes
  df$date_time <- as.factor(lubridate::ceiling_date(df$date_time, unit = "30 minutes"))

  # Aggregate data by date_time, calculating the mean for 'do' and 'temp'
  tidy_df <- stats::aggregate(cbind(do, temp) ~ date_time, df, function(x) mean(x, na.rm = TRUE))

  # Arrange the tidy dataframe by date_time
  tidy_df <- dplyr::arrange(tidy_df, date_time)

  # Convert date_time back to character and then to POSIXct format
  tidy_df$date_time <- as.character(tidy_df$date_time)
  tidy_df$date_time <- as.POSIXct(tidy_df$date_time, format = "%Y-%m-%d %H:%M:%S")

  # Set the timezone to Asia/Taipei
  tidy_df$date_time <- lubridate::force_tz(tidy_df$date_time, tzone = "Asia/Taipei")

  # Add the no_hobo parameter to the tidy dataframe
  tidy_df$no_hobo <- no_hobo

  # Return the cleaned and aggregated dataframe
  return(tidy_df)
}


#' @title convert_time
#' @importFrom readr read_csv
#' @importFrom tidyr fill
#' @importFrom lubridate as_datetime days force_tz
#' @description Tidy the daily weather data downloaded from weather station in Taiwan.
#' @param file_path Directory of file.
#' @param date Date of the daily weather data in yyyy-mm-dd format.
#' @param zone Code for the region of the weather station.
#' @return A dataframe.
#'@examples
#' weather_data_path <- system.file("extdata", "ex_weather.csv", package = "aelab")
#' df <- process_weather(weather_data_path, "2024-01-01", "site_A")
#' @export

process_weather <- function(file_path, date, zone) {

  # This function processes a CSV file downloaded with a specific format
  # Read the CSV file into a dataframe
  df <- readr::read_csv(file_path)

  # Identify the columns for hours, pressure (hPa), and wind speed (m/s)
  hours <- grep("\u89c0\u6e2c\u6642\u9593", names(df), ignore.case = F)  # "觀測時間"
  pressure_hpa <- grep("\u6e2c\u7ad9\u6c23", names(df), ignore.case = F)  # "測站氣"
  wind_speed <- grep("\u98a8\u901f", names(df), ignore.case = F)  # "風速"

  # Duplicate the dataframe and remove unnecessary rows and columns
  df <- rbind(df, df)  # Duplicate the dataframe
  df <- df[-c(1, 26), c(hours, pressure_hpa, wind_speed)]  # Remove the first row and the 26th row, keep relevant columns

  # Rename the columns for clarity
  colnames(df) <- c("hours", "pressure_hpa", "wind_ms")

  # Convert columns to numeric type
  df$hours <- as.numeric(df$hours)
  df$pressure_hpa <- as.numeric(df$pressure_hpa)
  df$wind_ms <- as.numeric(df$wind_ms)

  # Sort the dataframe by hours
  df <- df[order(df$hours), ]

  # Fill missing values in pressure and wind speed columns using down-up method
  df <- tidyr::fill(df, pressure_hpa, wind_ms, .direction = "downup")

  # Check if each hour from 1 to 24 has exactly 2 entries
  values <- 1:24
  results <- all(sapply(values, function(x) sum(df$hours == x) == 2))

  # If any hour is missing duplicates, issue a warning and return NULL
  if (!results) {
    warning("Some hours are missing duplicates in the dataset.")
    return(NULL)
  } else {
    # Create a sequence of time labels in 30-minute intervals
    time_sequence <- format(
      seq(
        from = as.POSIXct("00:30:00", format = "%H:%M:%S", tz = "Asia/Taipei"),
        by = "30 min",
        length.out = nrow(df)  # Length matches the number of rows in df
      ),
      format = "%H:%M:%S"
    )

    # Convert the input date to POSIXlt format
    df$date <- as.POSIXlt(date, format = "%Y-%m-%d", tz = "Asia/Taipei")

    # Add the time sequence to the dataframe
    df$time <- time_sequence

    # Combine date and time into a single datetime column
    df$date_time <- as.POSIXlt(paste(date, time_sequence), format = "%Y-%m-%d %H:%M:%S", tz = "Asia/Taipei")
    df$date_time <- as.POSIXct(df$date_time)  # Convert to POSIXct for easier manipulation
    df$date_time <- lubridate::force_tz(df$date_time, tzone = "Asia/Taipei")  # Ensure timezone is set correctly

    # Adjust the last entry's date_time to the next day
    new_date <- lubridate::as_datetime(df$date_time[48])
    df$date_time[48] <- new_date + lubridate::days(1)

    # Clean up unnecessary columns
    df$date <- NULL
    df$time <- NULL
    df$hours <- NULL
    df$zone <- zone  # Assign the zone parameter to the dataframe

    # Convert the dataframe to a standard data frame format and return it
    return(as.data.frame(df))
  }
}


#' @title process_info
#' @importFrom readxl read_excel
#' @importFrom lubridate force_tz
#' @importFrom dplyr mutate
#' @description Import and process the necessary information,
#' including the sunrise and sunset times of the day,
#' the date and time range of the deployment,
#' and the code for the data logger.
#' @param file_path Directory of file.
#' @return A dataframe.
#' @examples
#' info_data_path <- system.file("extdata", "info.xlsx", package = "aelab")
#' df <- process_info(info_data_path)
#' @export

process_info <- function(file_path) {

  # Read the Excel file into a dataframe
  info <- readxl::read_excel(file_path)

  # Convert sunrise, sunset, start_date_time, and end_date_time columns to Asia/Taipei timezone
  info$sunrise <- lubridate::force_tz(info$sunrise, tzone = "Asia/Taipei")
  info$sunset <- lubridate::force_tz(info$sunset, tzone = "Asia/Taipei")
  info$start_date_time <- lubridate::force_tz(info$start_date_time, tzone = "Asia/Taipei")
  info$end_date_time <- lubridate::force_tz(info$end_date_time, tzone = "Asia/Taipei")

  # Ensure no_hobo is treated as a character type
  info <- dplyr::mutate(info, no_hobo = as.character(no_hobo))

  # Return the processed dataframe
  return(info)
}

#' @title plot_hobo
#' @import ggplot2
#' @description Plot the dissolved oxygen concentration over time series grouped by different data loggers to observe the variations.
#' @param df Dataframe produced by process_hobo() function.
#' @return A plot generated by ggplot2.
#' @examples
#' data(hobo)
#' plot_hobo(hobo)
#' @export

plot_hobo <- function(df) {

  # Create a scatter plot using ggplot2
  ggplot2::ggplot(df, aes(x = date_time, y = do)) +
    geom_point(size = 1) +
    facet_grid(no_hobo ~ ., scales = "free")
}

#' @title calculate_do
#' @importFrom dplyr mutate filter summarise select
#' @description Calculate the Net Ecosystem Production,
#' Gross Primary Production and Ecosystem respiration based on the change in dissolved oxygen concentration.
#' @param df Merged dataframe produced by process_hobo(), process_weather() and process_info() functions.
#' @return A dataframe.
#' @examples
#' data(hobo)
#' calculate_do(hobo)
#' @export

calculate_do <- function(df) {

  # Calculate various metrics related to dissolved oxygen and ecosystem productivity
  df <- df |>
    dplyr::mutate(
      # Calculate daylight hours as the difference between sunset and sunrise
      daylight_hr = as.numeric(sunset - sunrise),

      # Extract time and date from date_time
      time = format(date_time, "%H:%M:%S"),
      date = as.Date(date_time, tz = "Asia/Taipei"),

      # Format sunset and sunrise times
      sunset_time = format(sunset, "%H:%M:%S"),
      sunrise_time = format(sunrise, "%H:%M:%S"),

      # Convert temperature from Celsius to Kelvin
      temp_k = temp + 273.15,

      # Calculate the concentration of oxygen (c_o2) using the temperature and salinity
      c_o2 = -173.4292 + 249.6336 * (100/temp_k) +
        143.3483 * log(temp_k/100) -
        21.8492 * (temp_k/100) +
        salinity * (-0.033096 + 0.014259 * (temp_k/100) - 0.0017 * (temp_k/100)^2),

      # Calculate oxygen saturation based on concentration
      o2_saturation = exp(c_o2) * 1.423,

      # Correct oxygen saturation pressure based on atmospheric pressure
      cor_o2_saturation_pressure = o2_saturation * (pressure_hpa * 0.0987 - 0.0112) / 100,

      # Calculate the rate of change in dissolved oxygen (DO)
      rate_do_change = (c(NA, diff(do))) * 2,  # Multiply by 2 to adjust for the time interval

      # Calculate Schmidt number (sc) based on temperature
      sc = 0.0476 * temp^3 + 3.7818 * temp^2 - 120.1 * temp + 1800.6,

      # Calculate gas exchange coefficient (k600) based on wind speed
      k600 = (2.07 + 0.215 * (wind_ms^1.7)) / 100,

      # Calculate the gas exchange coefficient (k) adjusted for Schmidt number
      k = k600 * (sc / 600)^(-0.5),

      # Calculate the flux of dissolved oxygen
      flux = (do - cor_o2_saturation_pressure) * k,

      # Calculate net ecosystem production (NEP) per hour
      nep_hr = rate_do_change * depth_m - flux
    )

  # Calculate average NEP for nighttime (outside of daylight hours)
  tidy_do_r <- df |>
    dplyr::filter(time < sunrise_time | time >= sunset_time) |>
    dplyr::summarise(r_hr = mean(nep_hr, na.rm = TRUE), .by = c(site, no_hobo, date))

  # Calculate average NEP for daytime (within daylight hours)
  tidy_do_nep <- df |>
    dplyr::filter(time >= sunrise_time, time < sunset_time) |>
    dplyr::summarise(
      nep_hr = mean(nep_hr, na.rm = TRUE),
      daylight_hr = mean(daylight_hr, na.rm = TRUE),
      .by = c(site, no_hobo, date)
    ) |>
    dplyr::mutate(nep_daytime = nep_hr * daylight_hr)

  # Merge nighttime and daytime NEP results
  tidy_do <- merge(tidy_do_r, tidy_do_nep, by = c("site", "no_hobo", "date"))

  # Calculate total respiration and gross primary production (GPP)
  tidy_do <- tidy_do |>
    dplyr::mutate(r_daytime = r_hr * daylight_hr,
                  r_day = r_hr * 24,  # Total respiration over 24 hours
                  gpp = nep_daytime - r_daytime,  # Gross primary production
                  nep = gpp + r_day  # Net ecosystem production
    ) |>
    dplyr::select(site, no_hobo, date, r_day, gpp, nep)  # Select relevant columns for output

  # Return the tidy dataframe with calculated metrics
  return(tidy_do)
}

#' @title combine_weather
#' @importFrom dplyr bind_rows
#' @description Tidy multiple daily weather data downloaded from weather station in Taiwan.
#' @param file_path Directory of folder containing the files (including the character in the file name that precedes the date).
#' @param start_date Date of the daily weather data in yyyy-mm-dd format.
#' @param end_date Date of the daily weather data in yyyy-mm-dd format.
#' @param zone Code for the region of the weather station.
#' @return A dataframe.
#'@examples
#' weather_data_path <- system.file("extdata", package = "aelab")
#' modified_data_path <- paste0(weather_data_path, "/ex_")
#' df <- combine_weather(modified_data_path,
#' start_date = "2024-01-01",
#' end_date = "2024-01-02", "site_A")
#' @export

combine_weather <- function(file_path, start_date, end_date, zone) {

  # Generate a sequence of dates from start_date to end_date
  dates <- as.character(seq(as.Date(start_date), as.Date(end_date), by = "day"))

  # Initialize an empty list to store data frames for each date
  df <- list()

  # Loop through each date to read and process the corresponding weather data
  for (date in dates) {
    # Construct the file name based on the date
    file_name <- paste0(file_path, date, ".csv")

    # Process the weather data for the current date and store it in the list
    df[[date]] <- process_weather(file_name, date, zone)
  }

  # Combine all data frames in the list into a single data frame
  weather <- dplyr::bind_rows(df)

  # Return the combined weather data
  return(weather)
}

#' @title combine_hobo
#' @importFrom stringr str_replace
#' @importFrom stringr str_remove
#' @importFrom purrr map_dfr
#' @description Tidy multiple data retrieved from HOBO U26 Dissolved Oxygen Data Logger.
#' @param file_path Directory of the folder containing the files.
#' @param file_prefix The prefix before the code for the data logger, defaults to "no."
#' @return A dataframe.
#' @examples
#' hobo_data_path <- system.file("extdata", package = "aelab")
#' df <- combine_hobo(hobo_data_path, file_prefix = "ex_ho")
#' @export

combine_hobo <- function(file_path, file_prefix = "no.") {

  # List all files in the specified directory that match the given prefix
  file_names <- list.files(file_path, pattern = paste0("^", file_prefix))

  # Use map_dfr to process all files and combine results (avoids O(n^2) rbind)
  df <- purrr::map_dfr(file_names, function(file_name) {
    # Extract the hobo number from the file name
    no_hobo <- stringr::str_replace(file_name, paste0("^", file_prefix), "")
    no_hobo <- stringr::str_remove(no_hobo, ".csv")

    # Construct the full file path for the current file
    file <- file.path(file_path, file_name)

    # Process the hobo data using the process_hobo function
    process_hobo(file_path = file, no_hobo = no_hobo)
  })

  # Return the combined data frame containing all processed hobo data
  return(df)
}


#' @title process_weather_month
#' @description Import and tidy a monthly weather CSV file downloaded from a
#'   Taiwan Central Weather Administration station. Column selection is done via
#'   regex so minor header changes are handled gracefully.
#' @param file_path Path to the monthly CSV file.
#' @param month Month number (1–12) covered by the file.
#' @param year Four-digit year. Default 2024.
#' @param zone Character label for the weather station / region.
#' @return A data frame with columns \code{day}, \code{pressure_hpa},
#'   \code{temp}, \code{humidity_percent}, \code{wind_ms}, \code{rain_mm},
#'   \code{daylight_hr}, \code{radiation}, \code{date}, and \code{zone}.
#' @examples
#' \dontrun{
#' df <- process_weather_month("path/to/2024-01.csv", month = 1, year = 2024,
#'                             zone = "site_A")
#' }
#' @importFrom readr read_csv
#' @importFrom dplyr mutate across where
#' @export
process_weather_month <- function(file_path, month, year = 2024, zone) {
  df <- readr::read_csv(file_path, show_col_types = FALSE)

  regex_patterns <- c(
    day              = "^\u89c0\u6e2c\u6642\u9593",   # 觀測時間
    pressure_hpa     = "^\u6e2c\u7ad9\u6c23\u58d3",  # 測站氣壓
    temp             = "^\u6c23\u6eab",               # 氣溫
    humidity_percent = "^\u76f8\u5c0d\u6fd5\u5ea6",  # 相對溼度
    wind_ms          = "^\u98a8\u901f",               # 風速
    rain_mm          = "^\u964d\u6c34\u91cf",         # 降水量
    daylight_hr      = "^\u65e5\u7167\u6642\u6578",   # 日照時數
    radiation        = "^\u5168\u5929\u7a7a\u65e5\u5c04\u91cf" # 全天空日射量
  )

  selected_cols <- sapply(regex_patterns, function(pattern) {
    match_cols <- grep(pattern, colnames(df), ignore.case = TRUE)
    if (length(match_cols) > 0) match_cols[1] else NULL
  })
  selected_cols <- unlist(selected_cols)

  df <- df[-1, selected_cols]
  colnames(df) <- names(selected_cols)

  df <- dplyr::mutate(df, dplyr::across(dplyr::where(is.character), as.numeric))
  df <- df[order(df$day), ]

  day_sequence <- seq(
    from       = as.Date(paste(year, month, 1, sep = "-"), format = "%Y-%m-%d"),
    by         = "1 day",
    length.out = nrow(df)
  )
  df$date <- as.Date(as.POSIXct(day_sequence, tz = "Asia/Taipei"))
  df$zone <- zone

  return(df)
}


#' @title combine_weather_month
#' @description Batch-import monthly weather CSV files from a Taiwan Central
#'   Weather Administration station for a consecutive range of months.
#' @details File names are expected to follow the pattern
#'   \code{<file_path><year>-0<month>.csv} (e.g. \code{2024-01.csv}).
#' @param file_path Path prefix (directory + filename prefix before the date
#'   portion, e.g. \code{"data/weather/"}).
#' @param start_month First month to import (1–9; two-digit months not yet
#'   supported).
#' @param end_month Last month to import.
#' @param year Four-digit year. Default 2024.
#' @param zone Character label for the weather station / region.
#' @return A combined data frame produced by \code{\link{process_weather_month}}.
#' @examples
#' \dontrun{
#' df <- combine_weather_month("data/weather/", start_month = 1,
#'                             end_month = 6, year = 2024, zone = "site_A")
#' }
#' @importFrom dplyr bind_rows
#' @importFrom purrr map_dfr
#' @export
combine_weather_month <- function(file_path, start_month, end_month,
                                  year = 2024, zone) {
  months <- as.character(seq(as.numeric(start_month), as.numeric(end_month)))

  purrr::map_dfr(months, function(month) {
    file_name <- paste0(file_path, year, "-0", month, ".csv")
    process_weather_month(file_name, month, year = year, zone = zone)
  })
}

test_that("process_hobo works correctly", {
  data <- process_hobo(test_file("ex_hobo.csv"), no_hobo = "test")

  expect_true(!is.null(data))
  column_names <- c("date_time", "do", "temp", "no_hobo")
  expect_identical(column_names, names(data))
  expect_identical("test", data$no_hobo[1])
  expect_true(all(!is.na(data$do)))
  expect_true(all(!is.na(data$temp)))
})

test_that("process_weather works correctly", {

  data <- process_weather(test_file("ex_weather.csv"), date = "2024-01-01", zone = "zone_A")

  column_names <- c("pressure_hpa", "wind_ms", "date_time", "zone")
  expect_identical(column_names, names(data))
  expect_true(is.numeric(data$pressure_hpa))
  expect_true(is.numeric(data$wind_ms))
  expect_true(inherits(data$date_time, "POSIXct"))
  expect_identical("zone_A", data$zone[1])
})

test_that("process_info works correctly", {

  data <- process_info(test_file("info.xlsx"))

  column_names <- c("zone", "site", "no_hobo", "depth_m",
                    "salinity", "start_date_time", "end_date_time", "sunrise", "sunset")
  expect_identical(column_names, names(data))
})

test_that("calculate_do calculates correctly", {
  df <- data.frame(
    date_time = as.POSIXct(c("2024-12-01 12:00:00", "2024-12-01 12:30:00", "2024-12-01 13:00:00"), tz = "Asia/Taipei"),
    do = c(8.5, 8.6, 8.7),
    temp = c(20, 21, 22),
    salinity = c(0, 0, 0),
    pressure_hpa = c(1013, 1014, 1015),
    wind_ms = c(2.5, 2.6, 2.7),
    sunrise = as.POSIXct("2024-12-01 06:00:00", tz = "Asia/Taipei"),
    sunset = as.POSIXct("2024-12-01 18:00:00", tz = "Asia/Taipei"),
    depth_m = c(1, 1, 1),
    site = c("site_A", "site_A", "site_A"),
    no_hobo  = "code_for_logger"
  )

  data <- calculate_do(df)

  expect_true("gpp" %in% colnames(data))  # Check if GPP is calculated
  expect_true("r_day" %in% colnames(data))  # Check if ER is calculated
  expect_true(all(!is.na(data$gpp)))
  expect_true(all(!is.na(data$r_day)))
})

---
title: "aelab"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{instruction}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(warning = FALSE)
knitr::opts_chunk$set(message = FALSE)
rmarkdown.html_vignette.check_title = FALSE
library(aelab)
library(readxl)
library(tibble)
library(lubridate)
library(stats)
library(dplyr)
library(openxlsx)
```

# Introduction

The goal of achieving net zero emissions by 2050 is a significant driver of research focused on understanding greenhouse gas (GHG) flux dynamics and the autotrophic production of ecosystems. This ambitious target highlights the urgent need to explore how different ecosystems contribute to GHG emissions and how they can be managed to reduce these emissions effectively. To support this research, this R package offers a variety of useful tools designed to facilitate the analysis of data related to GHG flux and production in aquatic environments.

# GHG flux calculation

Various techniques are used to measure greenhouse gas (GHG) flux, including eddy covariance (EC), the boundary layer method (BLM), and the static chamber method. This study provides R functions specifically designed to analyze data from GHG flux measurements using the static chamber method with a portable gas analyzer.
 
## Load and process the raw data file

First, we need to obtain necessary parameters from the raw data downloaded from the portable gas analyzer. We will remove unnecessary rows and columns and extract the GHG concentrations along with the corresponding date and time recorded by the analyzer using `tidy_ghg_analyzer()`. Additionally, this function will also eliminate any NaN values in the GHG data. Currently, this function supports raw data downloaded from:  
(i) LI-COR Trace Gas Analyzer (LI-7810 and LI-7820).
(ii) ABB LGR-ICOS Gas Analyzer (M-GGA-918).

```{r}
# The provided file is a raw data file downloaded from 
# the LI-COR Trace Gas Analyzer
ghg_data_path <- system.file("extdata", "ch4.xlsx", package = "aelab", mustWork = T)
ch4 <- tidy_ghg_analyzer(ghg_data_path, "ch4")
ch4[c(1:3), ]
```

Sometimes, the date and time recorded by the analyzer do not match the actual date and time we recorded in situ. Therefore, we need to convert the `date_time` column from the previous step to align with the real-life time, if there are any discrepancies:

```{r}
# The analyzer's time was assumed to be 
# 15 minutes and 30 seconds faster than real time
ch4 <- convert_time(ch4, min = -15, sec = 30)
ch4[c(1:3), c(5:6)]
```

## Calculate the slope

We first need to obtain the slope from the linear regression of GHG concentration changes over time within the chamber to calculate the flux. This can be done by defining the measurement time interval, either from a reference file (see Method 1 below) or directly in R (see Method 2 below).

### Method 1: Load from excel

In Excel, enter the date and time when the GHG flux measurement started, then load the file into R. Make sure the column containing the date and time information is formatted in ISO 8601 as `YYYY-MM-DD hh:mm:ss`.

```{r}
ref_data_path <- system.file("extdata", "reference.xlsx", package = "aelab", mustWork = T)
ref <- read_excel(ref_data_path)
ref
```

Now, we can perform multiple simple linear regressions on the GHG concentrations, separated by the `date_time` value in the `ref` data, using `calculate_regression()`. The measurement duration, referred to as `duration_minutes`, is set to a default of 7 minutes, and the number of rows used for the regression, also denoted as `duration_minutes`, defaults to 300 rows (which is equivalent to 5 minutes). These values can be modified if desired. 

When the measurement duration (specified by `duration_minutes`) exceeds the data used for regression (specified by `num_rows`), this function automatically selects the rows with the highest R$^2$ within the defined interval.

In the output tibble, `start_time` and `end_time` indicate the time range of the data used for the regression. `slope` represents the slope, and `r_square` denotes the R$^2$ value of the regression. `reference_time` is the start time of the measurement from the input.

```{r}
calculate_regression(ch4, ghg = "CH4", reference_time = ref$date_time,
                     duration_minutes = 7, num_rows = 300)
```

### Method 2: Type directly in R

The start time of measurement can also be input directly into the function. Please note that the reference_time must be in POSIXct format.

```{r}
calculate_regression(ch4, ghg = "CH4", reference_time = as.POSIXct("2023-03-11 07:32:00", tz = "UTC"))
```

## Calculate the flux

The equation we used to calculate the GHG flux was derived from Yong et al. (2024):

$$
F = \frac{(S \times V \times c)}{R \times T \times A}
$$

where S is the slope obtained from the linear regression of GHG concentration changes over time (ppm s$^{-1}$), V is the chamber volume (liters), c is the conversion factor from seconds to hours, R is the ideal gas constant (0.082 L atm K$^{−1}$ mol$^{−1}$), T is the temperature inside the chamber (kelvin), and A is the surface area of the chamber (m$^2$).

Therefore, the function `calculate_ghg_flux()` requires a dataframe with columns for the slope, area, volume, and temperature. Note that the units of the parameters must match those described above, resulting in a flux with units of mmol m$^{-2}$ d$^{-1}$.

```{r}
results_ch4 <- calculate_regression(ch4, ghg = "CH4", reference_time = as.POSIXct("2023-03-11 07:32:00", tz = "UTC"))
flux_ch4 <- data.frame(
    slope = results_ch4$slope,
    area = 1, # in square meter
    volume = 1, # in litre
    temp = 1) # in celcius
calculate_ghg_flux(flux_ch4)
```

## Additional GHG tools

### `convert_ghg_unit()`

`convert_ghg_unit()` converts a GHG flux value to µg m⁻² h⁻¹. The input can be a plain number or a character string containing embedded numbers (e.g., a mean ± SD string); each number in the string is converted individually.

```{r}
# Numeric input: 97 mg CH4 m-2 h-1 → µg m-2 h-1
convert_ghg_unit(97, ghg = "ch4", mass = "mg", area = "m2", time = "hr")

# Character string input: each number is converted individually
convert_ghg_unit("0.002 ± 0.003", ghg = "ch4", mass = "mmol", area = "m2", time = "hr")
```

### `calculate_MDF()`

`calculate_MDF()` calculates the Minimum Detectable Flux (MDF) for a static chamber system. This is useful for evaluating whether a gas analyser is precise enough to detect fluxes at the expected magnitude of the study site.

```{r}
calculate_MDF(
  precision_ppm     = 1,
  closure_time_s    = 300,
  data_point_n      = 300,
  chamber_volume_m3 = 0.0064,
  temperature_C     = 25,
  chamber_area_m2   = 0.07
)
```

### `calculate_total_co2e()`

`calculate_total_co2e()` aggregates individual GHG fluxes (mg m⁻² h⁻¹) into a single total CO₂-equivalent flux (g m⁻² d⁻¹) using IPCC AR6 100-year GWPs (CH₄ = 27, N₂O = 273).

```{r}
calculate_total_co2e(co2 = 4.02, ch4 = 0.001, n2o = 0.003)
```

# NEM calculation

The change in oxygen concentration in the water reflects the balance between photosynthetic production, respiratory consumption, and physical exchange at the water-atmosphere interface. By measuring dissolved oxygen (DO) concentrations at high frequency with in situ data loggers, we can calculate gross primary production (GPP), ecosystem respiration (ER), and net ecosystem metabolism (NEM), as detailed by Staehr et al. (2010).

## Load and process the raw data file

First, we need to obtain the necessary parameters from the raw data downloaded from the data loggers. We will remove unnecessary rows and columns and extract the dissolved oxygen concentrations and water temperature, along with the corresponding date and time recorded by the logger using `process_hobo()`. This function will also eliminate any NA values in the dataframe. The `no_hobo` input was necessary for further analysis. Currently, it supports raw data downloaded from the Onset HOBO Dissolved Oxygen Data Logger (U26-001).

```{r}
hobo_data_path <- system.file("extdata", "ex_hobo.csv", package = "aelab")
do <- process_hobo(hobo_data_path, no_hobo = "code_for_logger")
do[c(1:3), ]
```

The air pressure and wind speed at the sampling site should be obtained either manually or from a nearby weather station. The `process_weather` function tidies the CSV file downloaded from Taiwan's weather station. In addition to extracting the columns with air pressure and wind speed values, it also duplicates the hourly data to match the half-hourly recorded frequency of the DO concentrations.

```{r}
weather_data_path <- system.file("extdata", "ex_weather.csv", package = "aelab")
weather <- process_weather(weather_data_path, date = "2024-04-10", zone = "zone_A")
weather[c(1:5), ]
```

In addition to the previously mentioned parameters, we also needed the water depth, salinity, start and end date and time of the data logger deployment, and the sunrise and sunset times during that period. These parameters can be entered in Excel and then imported into R using `process_info()` to ensure that all necessary parameters are included. The column `zone` corresponds to `process_weather()`, and column `no_hobo` corresponds to `process_hobo()`.

```{r}
info_data_path <- system.file("extdata", "info.xlsx", package = "aelab")
info <- process_info(info_data_path)
info
```

Finally, we can merge the dataframe containing air pressure and wind speed data from `process_weather()` with the processed data downloaded from the data loggers. Next, we will combine this with the other necessary parameters using the `zone` and `no_hobo` columns. We can use the `plot_hobo()` function to inspect the changes in DO concentrations.

```{r}
data <- merge(weather, do, by = "date_time")
merged_df <- data %>% 
  inner_join(info, by = c("zone", "no_hobo"))
merged_df[c(1:3), ]
plot_hobo(merged_df)
```

## Calculate GPP and ER

GPP and ER can be calculated using `calculate_do()`. NEM (denoted as `nep` in the output dataframe) was calculated by GPP + ER.

```{r}
calculate_do(merged_df)
```

# Statistical analysis

aelab provides a small suite of statistical helpers that cover a typical aquatic ecology workflow: describe your data, check for outliers, test normality, transform if needed, and choose between a parametric or non-parametric group comparison.

The examples below use a simple self-contained data frame with three groups.

```{r}
set.seed(42)
stat_df <- data.frame(
  group = rep(c("A", "B", "C"), each = 8),
  value = c(rnorm(8, 2, 0.4), rnorm(8, 3.5, 0.5), rnorm(8, 5, 0.6))
)
```

## Descriptive statistics — `descriptive_statistic()`

`descriptive_statistic()` returns mean ± SD and min–max for each group.

```{r}
descriptive_statistic(stat_df, vars = value, groups = group)
```

## Outlier detection — `find_outlier()`

`find_outlier()` uses the IQR method (values outside 1.5 × IQR from Q1/Q3) to flag outliers.

```{r}
find_outlier(stat_df, var = "value")
```

## Normality testing

Use `normality_test_t()` when comparing two groups (t-test context), and `normality_test_aov()` when comparing three or more groups (ANOVA context). Both apply Shapiro-Wilk to raw, square-root, and log10 transforms so you can identify whether a transformation improves normality.

```{r}
# Two-group context
normality_test_t(stat_df, "value", group, "A", "B")

# One-way ANOVA context (all three groups)
normality_test_aov(stat_df, "value", "group")
```

## Data transformation — `df_trans()`

If residuals are not normal, `df_trans()` appends a transformed column to the data frame for re-testing.

```{r}
stat_df_t <- df_trans(stat_df, "value", "sqrt")
head(stat_df_t)
```

## Parametric test — `aov_test()`

When normality holds, `aov_test()` runs a one-way ANOVA followed by Tukey HSD post-hoc comparisons and returns compact letter display (CLD) groupings.

```{r}
aov_test(stat_df, "value", "group")
```

## Non-parametric test — `ks_test()`

When normality cannot be assumed, `ks_test()` runs Kruskal-Wallis followed by Dunn's post-hoc test with Bonferroni correction.

```{r}
ks_test(stat_df, "value", "group")
```

# Visualization

aelab provides plot wrappers — `plot_point()`, `plot_line()`, `plot_box()`, and `plot_bar()` — with a consistent theme built on Century Gothic. Custom colour palettes are available through `aelab_palettes()`, `scale_fill_aelab_d()`, and related scale functions.

The example below previews the `"ghg"` palette and applies it to a box plot comparing CH₄ fluxes across three habitat types.

```{r eval=FALSE}
# Preview the "ghg" palette
aelab_palettes("ghg")

# Box plot with aelab palette
df_vis <- data.frame(
  site = rep(c("Mangrove", "Mudflat", "Seagrass"), each = 10),
  ch4  = c(rnorm(10, 2, 0.5), rnorm(10, 1, 0.3), rnorm(10, 0.5, 0.2))
)

plot_box(df_vis, site, ch4, site) +
  scale_fill_aelab_d("ghg")
```

# Reference

Staehr, P. A., Bade, D., Van de Bogert, M. C., Koch, G. R., Williamson, C., Hanson, P., Cole, J. J., & Kratz, T. (2010). Lake metabolism and the diel oxygen technique: State of the science. Limnology and Oceanography: Methods, 8(11), 628–644. https://doi.org/10.4319/lom.2010.8.0628
Yong, Z.-J., Lin, W.-J., Lin, C.-W., & Lin, H.-J. (2024). Tidal influence on carbon dioxide and methane fluxes from tree stems and soils in mangrove forests. Biogeosciences, 21(22), 5247–5260. https://doi.org/10.5194/bg-21-5247-2024

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ghg_flux.R
\name{convert_ghg_unit}
\alias{convert_ghg_unit}
\title{convert_ghg_unit}
\usage{
convert_ghg_unit(
  input,
  ghg,
  mass = "µg",
  area = "m2",
  time = "hr",
  digits = 2,
  ratio = FALSE
)
}
\arguments{
\item{input}{A single numeric value or a character string containing one or
more numbers.}

\item{ghg}{The molecular formula of the greenhouse gas: \code{"co2"},
\code{"ch4"}, or \code{"n2o"}.}

\item{mass}{Mass unit of the input flux. One of \code{"mmol"}, \code{"mg"},
\code{"g"}, \code{"ug"} (micrograms), \code{"nmol"}, \code{"Mg"},
\code{"umol"} (micromoles), \code{"mol"}. Default \code{"ug"}.}

\item{area}{Area unit of the input flux. One of \code{"ha"}, \code{"m2"}.
Default \code{"m2"}.}

\item{time}{Time unit of the input flux. One of \code{"yr"}, \code{"day"},
\code{"hr"}, \code{"sec"}, \code{"min"}. Default \code{"hr"}.}

\item{digits}{Number of decimal places to round to. Default 2.}

\item{ratio}{Logical. If \code{TRUE}, apply an elemental-ratio correction
(C-basis for CH4, N-basis for N2O). Default \code{FALSE}.}
}
\value{
A named list with \code{value} (converted string) and \code{unit},
  or \code{"EMPTY"} for missing/non-numeric input.
}
\description{
Convert a greenhouse gas (GHG) flux value (or a character string
  containing one or more numeric values, e.g. \code{"0.002 +/- 0.003"})
  to micrograms per square meter per hour.
}
\details{
Numeric values embedded in a string (e.g. mean +/- SD notation)
  are each converted individually and the surrounding text is preserved.
  Commas are treated as decimal separators.
}
\examples{
convert_ghg_unit(97, ghg = "ch4", mass = "mg", area = "m2", time = "hr")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statistics.R
\name{normality_test_t}
\alias{normality_test_t}
\title{normality_test_t}
\usage{
normality_test_t(df, variable_name, group, group_1, group_2)
}
\arguments{
\item{df}{A data frame.}

\item{variable_name}{Name of the numeric variable column (string).}

\item{group}{<[`data-masking`][dplyr::dplyr_data_masking]> The grouping column.}

\item{group_1}{Value identifying the first group.}

\item{group_2}{Value identifying the second group.}
}
\value{
A tibble with Shapiro-Wilk p-values for each group × transformation
  combination.
}
\description{
Test normality of a variable within two groups using Shapiro-Wilk
  on raw, square-root, and log10 transforms (for t-test context).
}
\examples{
df <- data.frame(
  grp = c("A","A","A","B","B","B"),
  val = c(1.1, 2.0, 1.5, 4.2, 3.8, 4.5)
)
normality_test_t(df, "val", grp, "A", "B")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{scale_colour_aelab_c}
\alias{scale_colour_aelab_c}
\alias{scale_color_aelab_c}
\title{scale_colour_aelab_c}
\usage{
scale_colour_aelab_c(name, direction = 1)

scale_color_aelab_c(name, direction = 1)
}
\arguments{
\item{name}{Palette name passed to \code{\link{aelab_palettes}}.}

\item{direction}{\code{1} (default) for normal order; \code{-1} to reverse.}
}
\value{
A \code{ggplot2} scale.
}
\description{
Continuous ggplot2 colour scale using an aelab palette.
}
\examples{
\dontrun{
ggplot2::ggplot(mtcars, ggplot2::aes(wt, mpg, colour = mpg)) +
  ggplot2::geom_point() + scale_colour_aelab_c("rainbow")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statistics.R
\name{descriptive_statistic}
\alias{descriptive_statistic}
\title{descriptive_statistic}
\usage{
descriptive_statistic(data, vars, groups, digits = 2)
}
\arguments{
\item{data}{A data frame.}

\item{vars}{<[`tidy-select`][dplyr::dplyr_tidy_select]> Columns to summarise.}

\item{groups}{<[`tidy-select`][dplyr::dplyr_tidy_select]> Grouping columns.}

\item{digits}{Number of decimal places to round to. Default is 2.}
}
\value{
A tibble with one row per group and two summary columns per variable
  (`<var>_mean_sd` and `<var>_min_max`).
}
\description{
Compute grouped mean ± SD and min–max summary statistics for
  one or more numeric variables.
}
\examples{
df <- data.frame(group = c("A","A","B","B"), value = c(1.1, 2.3, 3.5, 4.7))
descriptive_statistic(df, vars = value, groups = group)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plot_point}
\alias{plot_point}
\title{plot_point}
\usage{
plot_point(
  df,
  x,
  y,
  z = NULL,
  base_size = 25,
  point_size = 3,
  stroke_size = 1,
  text_color = "black",
  facet = FALSE,
  facet_x = NULL,
  facet_y = NULL,
  style = "bw"
)
}
\arguments{
\item{df}{A data frame.}

\item{x}{<[`data-masking`][ggplot2::aes]> Column mapped to the x-axis.}

\item{y}{<[`data-masking`][ggplot2::aes]> Column mapped to the y-axis.}

\item{z}{<[`data-masking`][ggplot2::aes]> Optional column mapped to fill colour.}

\item{base_size}{Base font size passed to the ggplot2 theme. Default 25.}

\item{point_size}{Point size. Default 3.}

\item{stroke_size}{Point stroke width. Default 1.}

\item{text_color}{Text colour. Default \code{"black"}.}

\item{facet}{Logical; add facet grid? Default \code{FALSE}.}

\item{facet_x}{Column name (string) for the horizontal facet dimension.}

\item{facet_y}{Column name (string) for the vertical facet dimension.}

\item{style}{Theme style. One of \code{"bw"}, \code{"minimal"},
\code{"classic"}, \code{"graycolor"}, \code{"light"}. Default \code{"bw"}.}
}
\value{
A \code{ggplot} object.
}
\description{
Create a scatter plot using the aelab theme.
}
\examples{
\dontrun{
df <- data.frame(x = 1:5, y = c(2,4,1,5,3), g = c("A","A","B","B","A"))
plot_point(df, x, y, g)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hobo_do.R
\name{plot_hobo}
\alias{plot_hobo}
\title{plot_hobo}
\usage{
plot_hobo(df)
}
\arguments{
\item{df}{Dataframe produced by process_hobo() function.}
}
\value{
A plot generated by ggplot2.
}
\description{
Plot the dissolved oxygen concentration over time series grouped by different data loggers to observe the variations.
}
\examples{
data(hobo)
plot_hobo(hobo)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ghg_flux.R
\name{calculate_MDF}
\alias{calculate_MDF}
\title{calculate_MDF}
\usage{
calculate_MDF(
  precision_ppm,
  closure_time_s,
  data_point_n,
  chamber_volume_m3,
  temperature_C,
  chamber_area_m2,
  pressure_pa = 101325,
  ideal_constant = 8.314,
  ghg = "co2"
)
}
\arguments{
\item{precision_ppm}{Precision of the gas analyser (ppm).}

\item{closure_time_s}{Closure time of the measurement (seconds).}

\item{data_point_n}{Number of data points recorded during the closure period.}

\item{chamber_volume_m3}{Internal volume of the chamber (m\eqn{^3}).}

\item{temperature_C}{Air temperature at the measurement location (\eqn{^\circ}C).}

\item{chamber_area_m2}{Base area of the chamber (m\eqn{^2}).}

\item{pressure_pa}{Atmospheric pressure (Pa). Default 101325.}

\item{ideal_constant}{Ideal gas constant (J mol\eqn{^{-1}} K\eqn{^{-1}}).
Default 8.314.}

\item{ghg}{Greenhouse gas type: \code{"co2"}, \code{"ch4"}, or \code{"n2o"}.
Default \code{"co2"}.}
}
\value{
A named list with \code{MDF} (numeric,
  \eqn{\mu}g m\eqn{^{-2}} h\eqn{^{-1}}) and \code{unit} (string).
}
\description{
Calculate the Minimum Detectable Flux (MDF) for a static chamber
  GHG measurement system.
}
\examples{
calculate_MDF(
  precision_ppm     = 1,
  closure_time_s    = 300,
  data_point_n      = 300,
  chamber_volume_m3 = 0.0064,
  temperature_C     = 25,
  chamber_area_m2   = 0.07
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ghg_flux.R
\name{convert_time}
\alias{convert_time}
\title{convert_time}
\usage{
convert_time(data, day = 0, hr = 0, min = 0, sec = 0)
}
\arguments{
\item{data}{Data from the LI-COR Trace Gas Analyzer that had been processed by tidy_licor().}

\item{day}{Day(s) to add or subtract.}

\item{hr}{Hour(s) to add or subtract.}

\item{min}{Minute(s) to add or subtract.}

\item{sec}{Second(s) to add or subtract.}
}
\value{
The input data with a new column in POSIXct format converted based on the input value.
}
\description{
Convert the time of the LI-COR Trace Gas Analyzer to match the time in real life.
}
\examples{
data(n2o)
converted_n2o <- convert_time(n2o, min = -10, sec = 5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hobo_do.R
\name{process_info}
\alias{process_info}
\title{process_info}
\usage{
process_info(file_path)
}
\arguments{
\item{file_path}{Directory of file.}
}
\value{
A dataframe.
}
\description{
Import and process the necessary information,
including the sunrise and sunset times of the day,
the date and time range of the deployment,
and the code for the data logger.
}
\examples{
info_data_path <- system.file("extdata", "info.xlsx", package = "aelab")
df <- process_info(info_data_path)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hobo.R
\name{hobo}
\alias{hobo}
\title{Processed data from Onset HOBO Dissolved Oxygen Data Logger.
A dataset containing 336 dissolved oxygen concentrations changed over time.}
\format{
A data.frame with 336 rows and 13 variables:
\itemize{
  \item date_time: Date and time in POSIXct format.
  \item pressure_hpa: Atmospheric pressure (hpa).
  \item wind_ms: Wind speed (m/s).
  \item do: Dissolved oxygen concentrations (mg/L)
  \item temp: Water temperature (Celsius)
  \item depth: Water depth (m).
  \item salinity: Salinity (ppt).
  \item start_date_time: Start date and time of the deployment.
  \item end_date_time: End date and time of the deployment.
  \item sunrise: Sunrise time during that day.
  \item sunset: Sunset time during that day.
  \item no_hobo: Name for the data logger .
  \item site: Name for the site.
}
}
\source{
own data.
}
\description{
Processed data from Onset HOBO Dissolved Oxygen Data Logger.
A dataset containing 336 dissolved oxygen concentrations changed over time.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statistics.R
\name{ks_test}
\alias{ks_test}
\title{ks_test}
\usage{
ks_test(df, variable_name, group)
}
\arguments{
\item{df}{A data frame.}

\item{variable_name}{Name of the response variable column (string).}

\item{group}{Name of the grouping column (string).}
}
\value{
A named list with elements \code{ks_results}, \code{dunn_results},
  \code{mean_summary}, and \code{compact_letters}.
}
\description{
Perform Kruskal-Wallis test followed by Dunn post-hoc test
  (Bonferroni correction) with compact letter display.
}
\examples{
df <- data.frame(
  grp = rep(c("A","B","C"), each = 5),
  val = c(1,2,1,2,1, 3,4,3,4,3, 5,6,5,6,5)
)
ks_test(df, "val", "grp")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ghg_flux.R
\name{calculate_total_co2e}
\alias{calculate_total_co2e}
\title{calculate_total_co2e}
\usage{
calculate_total_co2e(co2 = 0, ch4 = 0, n2o = 0)
}
\arguments{
\item{co2}{CO\eqn{_2} flux in mg m\eqn{^{-2}} h\eqn{^{-1}}. Default 0.}

\item{ch4}{CH\eqn{_4} flux in mg m\eqn{^{-2}} h\eqn{^{-1}}. Default 0.}

\item{n2o}{N\eqn{_2}O flux in mg m\eqn{^{-2}} h\eqn{^{-1}}. Default 0.}
}
\value{
Total CO\eqn{_2}e flux as a numeric scalar
  (g m\eqn{^{-2}} d\eqn{^{-1}}), printed with a diagnostic message.
}
\description{
Convert individual GHG fluxes (mg m\eqn{^{-2}} h\eqn{^{-1}})
  to a total CO\eqn{_2}-equivalent flux (g m\eqn{^{-2}} d\eqn{^{-1}}) using
  IPCC AR6 100-year GWPs (CO\eqn{_2} = 1, CH\eqn{_4} = 27,
  N\eqn{_2}O = 273).
}
\examples{
calculate_total_co2e(co2 = 4.02, ch4 = 0.001, n2o = 0.003)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statistics.R
\name{df_trans}
\alias{df_trans}
\title{df_trans}
\usage{
df_trans(df, variable_name, transformation)
}
\arguments{
\item{df}{A data frame.}

\item{variable_name}{Name of the column to transform (string).}

\item{transformation}{Transformation type: \code{"sqrt"} or \code{"log"}.}
}
\value{
The input data frame with an additional column named
  \code{<variable_name>_sqrt} or \code{<variable_name>_log}.
}
\description{
Apply a reverse square-root or reverse log transformation to a
  numeric column and append the result as a new column.
}
\examples{
df <- data.frame(val = c(1, 4, 9, 16))
df_trans(df, "val", "sqrt")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hobo_do.R
\name{process_weather_month}
\alias{process_weather_month}
\title{process_weather_month}
\usage{
process_weather_month(file_path, month, year = 2024, zone)
}
\arguments{
\item{file_path}{Path to the monthly CSV file.}

\item{month}{Month number (1–12) covered by the file.}

\item{year}{Four-digit year. Default 2024.}

\item{zone}{Character label for the weather station / region.}
}
\value{
A data frame with columns \code{day}, \code{pressure_hpa},
  \code{temp}, \code{humidity_percent}, \code{wind_ms}, \code{rain_mm},
  \code{daylight_hr}, \code{radiation}, \code{date}, and \code{zone}.
}
\description{
Import and tidy a monthly weather CSV file downloaded from a
  Taiwan Central Weather Administration station. Column selection is done via
  regex so minor header changes are handled gracefully.
}
\examples{
\dontrun{
df <- process_weather_month("path/to/2024-01.csv", month = 1, year = 2024,
                            zone = "site_A")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hobo_do.R
\name{calculate_do}
\alias{calculate_do}
\title{calculate_do}
\usage{
calculate_do(df)
}
\arguments{
\item{df}{Merged dataframe produced by process_hobo(), process_weather() and process_info() functions.}
}
\value{
A dataframe.
}
\description{
Calculate the Net Ecosystem Production,
Gross Primary Production and Ecosystem respiration based on the change in dissolved oxygen concentration.
}
\examples{
data(hobo)
calculate_do(hobo)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lab_analysis.R
\name{calc_chla_trichromatic}
\alias{calc_chla_trichromatic}
\title{calc_chla_trichromatic}
\usage{
calc_chla_trichromatic(wl_630, wl_647, wl_664, wl_750)
}
\arguments{
\item{wl_630}{Absorbance at 630 nm.}

\item{wl_647}{Absorbance at 647 nm.}

\item{wl_664}{Absorbance at 664 nm.}

\item{wl_750}{Absorbance at 750 nm (turbidity blank).}
}
\value{
Chlorophyll-a concentration in µg L\eqn{^{-1}} (assuming a 1 cm path
  length and standard extraction volume).
}
\description{
Calculate chlorophyll-a concentration from trichromatic
  spectrophotometric absorbance readings using the Jeffrey & Humphrey (1975)
  equations.
}
\details{
Absorbance values should be measured in a 1 cm path-length cuvette.
  The 750 nm reading is used as a turbidity blank correction.
  Formula: \eqn{11.85 \times E_{664} - 1.54 \times E_{647} - 0.08 \times E_{630}}
  where \eqn{E_{\lambda} = A_{\lambda} - A_{750}}.
}
\examples{
calc_chla_trichromatic(wl_630 = 0.05, wl_647 = 0.08, wl_664 = 0.20, wl_750 = 0.01)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ghg_flux.R
\name{tidy_ghg_analyzer}
\alias{tidy_ghg_analyzer}
\title{tidy_ghg_analyzer}
\usage{
tidy_ghg_analyzer(file_path, gas, analyzer = "licor")
}
\arguments{
\item{file_path}{Directory of file.}

\item{gas}{Choose between CO2/CH4 or N2O LI-COR Trace Gas Analyzer, which is "ch4" and "n2o", respectively.}

\item{analyzer}{The brand of the analyzer which the data was downloaded from.}
}
\value{
Return the loaded XLSX file after tidying for further analysis.
}
\description{
Tidy the data downloaded from GHG Analyzer.
}
\examples{
ghg_data_path <- system.file("extdata", "ch4.xlsx", package = "aelab")
tidy_ghg_analyzer(ghg_data_path, "ch4")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{aelab_palettes}
\alias{aelab_palettes}
\title{aelab_palettes}
\usage{
aelab_palettes(name, n, type = c("discrete", "continuous"))
}
\arguments{
\item{name}{Name of the palette (string).}

\item{n}{Number of colours to return. Defaults to the full palette length.}

\item{type}{\code{"discrete"} (default) or \code{"continuous"}. For
\code{"continuous"}, colours are interpolated to length \code{n}.}
}
\value{
A character vector of hex colour codes with class \code{"palette"}.
}
\description{
Retrieve a named aelab colour palette as a character vector.
}
\details{
Available palette names: \code{"rainbow"}, \code{"two"},
  \code{"control"}, \code{"control2"}, \code{"control3"},
  \code{"period"}, \code{"ghg"}.
}
\examples{
aelab_palettes("rainbow", 5)
aelab_palettes("ghg", type = "continuous", n = 20)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/n2o.R
\name{n2o}
\alias{n2o}
\title{Processed data from N2O LI-COR Trace Gas Analyzer.
A dataset containing 567 N2O concentrations changed over time.}
\format{
A data.frame with 567 rows and 4 variables:
\itemize{
  \item DATE: Date in character format.
  \item TIME: Time in character format.
  \item N2O: Concentrations of nitrous oxide (N2O), in ppb.
  \item date_time: Date and time in POSIXct format.
}
}
\source{
own data.
}
\description{
Processed data from N2O LI-COR Trace Gas Analyzer.
A dataset containing 567 N2O concentrations changed over time.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statistics.R
\name{normality_test_aov}
\alias{normality_test_aov}
\title{normality_test_aov}
\usage{
normality_test_aov(df, variable_name, group_1, group_2 = NULL)
}
\arguments{
\item{df}{A data frame.}

\item{variable_name}{Name of the response variable column (string).}

\item{group_1}{Name of the first grouping column (string).}

\item{group_2}{Name of the second grouping column (string), or \code{NULL}
for a one-way model.}
}
\value{
A tibble with Shapiro-Wilk p-values for each transformation.
}
\description{
Test normality of ANOVA model residuals using Shapiro-Wilk on
  raw, square-root, and log10 transforms (one-way or two-way).
}
\examples{
df <- data.frame(
  grp = c("A","A","B","B"),
  val = c(1.1, 1.4, 3.2, 3.8)
)
normality_test_aov(df, "val", "grp")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ghg_flux.R
\name{calculate_ghg_flux}
\alias{calculate_ghg_flux}
\title{calculate_ghg_flux}
\usage{
calculate_ghg_flux(
  data,
  slope = "slope",
  area = "area",
  volume = "volume",
  temp = "temp"
)
}
\arguments{
\item{data}{A data frame containing relevant data with columns for slope, area, volume, and temperature.}

\item{slope}{Name of the column in `data` that contains the slope values of the GHG concentration change (in ppm/s).}

\item{area}{Name of the column in `data` that contains the values of the area of the chamber (in square meter).}

\item{volume}{Name of the column in `data` that contains values of the volume of the chamber (in litre).}

\item{temp}{Name of the column in `data` that contains values of the temperature of the gas (in Celsius).}
}
\value{
A list containing the calculated flux and its unit.
}
\description{
Calculate the greenhouse gas (GHG) flux based on input parameters from a data frame.
}
\examples{
data <- data.frame(
  slope = c(1.2, 1.5, 1.1),
  area = c(100, 150, 120),
  volume = c(10, 15, 12),
  temp = c(25, 30, 22)
)
results <- calculate_ghg_flux(data)
print(results)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ghg_flux.R
\name{calculate_regression}
\alias{calculate_regression}
\title{calculate_regression}
\usage{
calculate_regression(
  data,
  reference_df,
  ghg,
  reference_time,
  site,
  analyzer_code,
  duration_minutes = 7,
  num_rows = 300
)
}
\arguments{
\item{data}{Data from the GHG analyzer that has been processed and time-converted.}

\item{reference_df}{A data frame containing measurement reference times, site names, and an \code{analyzer} column.}

\item{ghg}{Column name in \code{data} containing GHG concentration values (e.g., \code{"CH4"}, \code{"N2O"}).}

\item{reference_time}{Column name in \code{reference_df} containing the measurement start times (POSIXct).}

\item{site}{Column name in \code{reference_df} containing site identifiers.}

\item{analyzer_code}{String pattern used to filter results by the \code{analyzer} column of \code{reference_df}.}

\item{duration_minutes}{The duration of the measurement, default to 7.}

\item{num_rows}{The number of rows used to perform the regression, default to 300.}
}
\value{
A tibble containing the time range, slope, R2, site, and analyzer from the simple linear regression.
}
\description{
Calculate the slope of greenhouse gas (GHG) concentration change over time using simple linear regression.
}
\examples{
data(n2o)
n2o_converted <- convert_time(n2o)
ref <- data.frame(
  date_time = n2o_converted$real_datetime[1],
  site = "S1",
  analyzer = "1074"
)
calculate_regression(n2o_converted, ref, "N2O",
                     reference_time = "date_time", site = "site",
                     analyzer_code = "1074")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hobo_do.R
\name{combine_hobo}
\alias{combine_hobo}
\title{combine_hobo}
\usage{
combine_hobo(file_path, file_prefix = "no.")
}
\arguments{
\item{file_path}{Directory of the folder containing the files.}

\item{file_prefix}{The prefix before the code for the data logger, defaults to "no."}
}
\value{
A dataframe.
}
\description{
Tidy multiple data retrieved from HOBO U26 Dissolved Oxygen Data Logger.
}
\examples{
hobo_data_path <- system.file("extdata", package = "aelab")
df <- combine_hobo(hobo_data_path, file_prefix = "ex_ho")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statistics.R
\name{sig_labels}
\alias{sig_labels}
\title{sig_labels}
\usage{
sig_labels(
  stat_data,
  variable,
  group,
  by = "year",
  plot_data = NULL,
  alpha = 0.05
)
}
\arguments{
\item{stat_data}{Data frame of raw observations used for the statistical
test. Should already be filtered to the relevant subset (e.g. a single
water type).}

\item{variable}{Name of the response variable column (string).}

\item{group}{Name of the grouping column (string).}

\item{by}{Name of the faceting column whose levels are iterated over
(string, default \code{"year"}).}

\item{plot_data}{Data frame used to compute label y-positions via
\code{max(variable)} per group. Defaults to \code{stat_data}. Pass the
aggregated/boxplot data here when it differs from the raw test data.}

\item{alpha}{Significance threshold; facets with KW p-value >=
\code{alpha} are dropped (default \code{0.05}).}
}
\value{
A data frame with columns \code{<group>}, \code{Letter},
  \code{MonoLetter}, \code{y_pos}, and \code{<by>}. Returns an empty data
  frame when no facet reaches significance.
}
\description{
Run \code{\link{ks_test}} separately for each level of a
  faceting variable and return compact letter display (CLD) annotations
  ready for \code{geom_text}. Only facets where the Kruskal-Wallis p-value
  is below \code{alpha} are included in the output.
}
\examples{
set.seed(1)
df <- data.frame(
  year = rep(c("2023", "2024"), each = 20),
  grp  = rep(c("A","B","C","D"), 10),
  val  = c(rnorm(20, mean = rep(c(1,3,2,6), 5)), rnorm(20))
)
sig_labels(df, "val", "grp", by = "year")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hobo_do.R
\name{combine_weather_month}
\alias{combine_weather_month}
\title{combine_weather_month}
\usage{
combine_weather_month(file_path, start_month, end_month, year = 2024, zone)
}
\arguments{
\item{file_path}{Path prefix (directory + filename prefix before the date
portion, e.g. \code{"data/weather/"}).}

\item{start_month}{First month to import (1–9; two-digit months not yet
supported).}

\item{end_month}{Last month to import.}

\item{year}{Four-digit year. Default 2024.}

\item{zone}{Character label for the weather station / region.}
}
\value{
A combined data frame produced by \code{\link{process_weather_month}}.
}
\description{
Batch-import monthly weather CSV files from a Taiwan Central
  Weather Administration station for a consecutive range of months.
}
\details{
File names are expected to follow the pattern
  \code{<file_path><year>-0<month>.csv} (e.g. \code{2024-01.csv}).
}
\examples{
\dontrun{
df <- combine_weather_month("data/weather/", start_month = 1,
                            end_month = 6, year = 2024, zone = "site_A")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plot_map_taiwan}
\alias{plot_map_taiwan}
\title{plot_map_taiwan}
\usage{
plot_map_taiwan(
  long,
  lat,
  names,
  color = "darkgrey",
  textsize = 5,
  basesize = 16,
  shape_type = 22
)
}
\arguments{
\item{long}{Numeric vector of longitudes.}

\item{lat}{Numeric vector of latitudes.}

\item{names}{Character vector of site labels (same length as \code{long}).}

\item{color}{Fill colour for site markers. Default \code{"darkgrey"}.}

\item{textsize}{Size for annotation and point labels. Default 5.}

\item{basesize}{Base font size for the map theme. Default 16.}

\item{shape_type}{ggplot2 point shape number. Default 22 (filled square).}
}
\value{
A \code{ggplot} object (also printed to the active device).
}
\description{
Plot sampling sites on a map of Taiwan with a north arrow and
  scale bar.
}
\examples{
\dontrun{
plot_map_taiwan(
  long = c(120.2, 121.5),
  lat  = c(22.9, 24.1),
  names = c("Site A", "Site B")
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statistics.R
\name{find_outlier}
\alias{find_outlier}
\title{find_outlier}
\usage{
find_outlier(df, var, other_var = NULL)
}
\arguments{
\item{df}{A data frame.}

\item{var}{Name of the column to check for outliers (string).}

\item{other_var}{Character vector of additional column names to return
alongside the outlier values, or \code{NULL}.}
}
\value{
A tibble with columns \code{row_index}, \code{outlier_value}, and any
  requested \code{other_var} columns.
}
\description{
Identify outliers in a numeric column using the IQR method
  (values outside 1.5 \eqn{\times} IQR from Q1/Q3).
}
\examples{
df <- data.frame(val = c(1, 2, 2, 3, 100), id = 1:5)
find_outlier(df, "val", "id")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statistics.R
\name{aov_test}
\alias{aov_test}
\title{aov_test}
\usage{
aov_test(df, variable_name, group)
}
\arguments{
\item{df}{A data frame.}

\item{variable_name}{Name of the response variable column (string).}

\item{group}{Name of the grouping column (string).}
}
\value{
A named list with elements \code{anova_summary}, \code{tukey_results},
  and \code{compact_letters}.
}
\description{
Perform one-way ANOVA followed by Tukey HSD post-hoc test with
  compact letter display.
}
\examples{
df <- data.frame(
  grp = rep(c("A","B","C"), each = 5),
  val = c(1,2,1,2,1, 3,4,3,4,3, 5,6,5,6,5)
)
aov_test(df, "val", "grp")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hobo_do.R
\name{process_hobo}
\alias{process_hobo}
\title{process_hobo}
\usage{
process_hobo(file_path, no_hobo)
}
\arguments{
\item{file_path}{Directory of file.}

\item{no_hobo}{The code for the data logger.}
}
\value{
A dataframe.
}
\description{
Tidy the data retrieved from HOBO U26 Dissolved Oxygen Data Logger.
}
\examples{
hobo_data_path <- system.file("extdata", "ex_hobo.csv", package = "aelab")
df <- process_hobo(hobo_data_path, "code_for_logger")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hobo_do.R
\name{combine_weather}
\alias{combine_weather}
\title{combine_weather}
\usage{
combine_weather(file_path, start_date, end_date, zone)
}
\arguments{
\item{file_path}{Directory of folder containing the files (including the character in the file name that precedes the date).}

\item{start_date}{Date of the daily weather data in yyyy-mm-dd format.}

\item{end_date}{Date of the daily weather data in yyyy-mm-dd format.}

\item{zone}{Code for the region of the weather station.}
}
\value{
A dataframe.
}
\description{
Tidy multiple daily weather data downloaded from weather station in Taiwan.
}
\examples{
weather_data_path <- system.file("extdata", package = "aelab")
modified_data_path <- paste0(weather_data_path, "/ex_")
df <- combine_weather(modified_data_path,
start_date = "2024-01-01",
end_date = "2024-01-02", "site_A")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hobo_do.R
\name{process_weather}
\alias{process_weather}
\title{convert_time}
\usage{
process_weather(file_path, date, zone)
}
\arguments{
\item{file_path}{Directory of file.}

\item{date}{Date of the daily weather data in yyyy-mm-dd format.}

\item{zone}{Code for the region of the weather station.}
}
\value{
A dataframe.
}
\description{
Tidy the daily weather data downloaded from weather station in Taiwan.
}
\examples{
weather_data_path <- system.file("extdata", "ex_weather.csv", package = "aelab")
df <- process_weather(weather_data_path, "2024-01-01", "site_A")
}
